/// ---- file     : tkfox.cpp
/// ---- copyright: (c)2004 by Bastian Spiegel <bs@tkscript.de>
/// ---- license  : publically available under terms of the GNU General Public License, see "COPYING".
/// ---- depends  : ..on the FOX toolkit <http://www.fox-toolkit.org> (and TKScript)
/// ----
/// ---- note     : I also dislike multiple inheritance in C++ but there was no alternative.
/// ----            I could have adapted the entire fox library to tkscript but this would
/// ----            have caused a real nightmare every time FOX gets updated.
/// ----            Please read http://tkscript.de/tkfox.html for a brief introduction to FXML.
/// ----           
/// ----            Happy hacking!
/// ----

#define YAC_PRINTF
#define YAC_BIGSTRING

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include <yac.h>
#include <yac_host.cpp>
YAC_Host *yac_host;

#include <fx.h>
#include "fx3d.h"
#include <GL/gl.h>

FXApp *application;
FXString    sfx_unset;

FXGLVisual *fxglvisual;

FXGLVisual *GetFXGLVisual(void) {
	if(!fxglvisual)
		fxglvisual=new FXGLVisual(application,VISUAL_DOUBLEBUFFER|VISUAL_STEREO);
	//yac_host->printf("fxglvisual=%08x\n",(sUI)fxglvisual);
	return fxglvisual;
}

#define TKS_FXSTACKSIZE 512
#define TKS_FXRELOCSIZE 2048
#define TKS_FXMAXSCRIPTS 128

class TKS_FXMainWindow;
class TKS_FXButton;
class TKS_FXTextField;
class TKS_FXSlider;
class TKS_FXMenuCommand;
class TKS_FXDial;
class TKS_FXScrollBar;
class TKS_FXGLCanvas;
class TKS_FXLabel;
class TKS_FXMessageBox;
class TKS_FXFileDialog;
class TKS_FX4Splitter;
class TKS_FXColorDialog;
class TKS_FXCheckButton;
class TKS_FXComboBox;

typedef union __cobjectptr {
		    YAC_Object            *yacobject;
			FXObject              *fxobject;
			FXWindow              *fxwindow;
			FXComposite           *fxcomposite;
			FXFrame               *fxframe;
			FXFont                *fxfont;
			FXMenuBar             *fxmenubar;
			FXMenuPane            *fxmenupane;
			FXMenuTitle           *fxmenutitle;
			TKS_FXMenuCommand     *tksfxmenucommand;
			//TKS_FXMainWindow      *tksfxcomposite;
			TKS_FXMainWindow      *tksfxmainwindow;
			TKS_FXButton          *tksfxbutton;
			TKS_FXTextField       *tksfxtextfield;
			TKS_FXSlider          *tksfxslider;
			FXHorizontalFrame     *fxhorizontalframe;
			FXVerticalFrame       *fxverticalframe;
			TKS_FXDial            *tksfxdial;
			TKS_FXScrollBar       *tksfxscrollbar;
			FXGroupBox            *fxgroupbox;
			FXHorizontalSeparator *fxhorizontalseparator;
			FXVerticalSeparator   *fxverticalseparator;
			TKS_FXGLCanvas        *tksfxglcanvas;
			TKS_FXLabel           *tksfxlabel;
			FXLabel               *fxlabel;
			TKS_FXMessageBox      *tksfxmessagebox;
			TKS_FXFileDialog      *tksfxfiledialog;
			TKS_FX4Splitter       *tksfx4splitter;
			TKS_FXColorDialog     *tksfxcolordialog;
			TKS_FXCheckButton     *tksfxcheckbutton;
			TKS_FXComboBox        *tksfxcombobox;
			
			sUI               sui;
} cobjectptr;

sUI                                clid_FXPage;
sUI                                clid_FXMainWindow;
sUI                                clid_FXLabel;
sUI                                clid_FXButton;
sUI                                clid_FXDial;
sUI                                clid_FXSlider;
sUI                                clid_FXScrollBar;
sUI                                clid_FXTextField;
sUI                                clid_FXMessageBox;
sUI                                clid_FXFileDialog;
sUI                                clid_FX4Splitter;
sUI                                clid_FXColorDialog;
sUI                                clid_FXCheckButton;
sUI                                clid_FXComboBox;

sBool TKS_FXIsComposite(YAC_Object *_o) {
	return 1;
		/*
		(_o->class_ID==clid_FXMainWindow)     ||
		(_o->class_ID==clid_FXHorizontalFrame)||
		(_o->class_ID==clid_FXVerticalFrame)  ||
		(_o->class_ID==clid_FXGroupBox)       ||
		(_o->class_ID==clid_FXGLCanvas)       ;
		*/
}

YAC_String cb_odecl_fxname;
YAC_String cb_odecl_fxprefix;
YAC_String cb_odecl_fxprefix2;
YAC_String cb_odecl_fxsuffix;
YAC_String cb_odecl_fxbutton; 
YAC_String cb_odecl_fxdial; 
YAC_String cb_odecl_fxslider; 
YAC_String cb_odecl_fxscrollbar; 
YAC_String cb_odecl_fxtextfield; 
YAC_String cb_odecl_fxglcanvas;
YAC_String cb_odecl_fxlabel;
YAC_String cb_odecl_fxmessagebox;
YAC_String cb_odecl_fxmainwindow;
YAC_String cb_odecl_fxfiledialog;
YAC_String cb_odecl_fx4splitter;
YAC_String cb_odecl_fxcolordialog;
YAC_String cb_odecl_fxcheckbutton;
YAC_String cb_odecl_fxcombobox;

YAC_String cb_args_fxtextfield_onVerify;
YAC_String cb_args_fxtextfield_onKey;
YAC_String cb_args_fxslider_onChange;
YAC_String cb_args_fxglcanvas_onMotion;
YAC_String cb_args_fxcheckbutton_onLeftBtnRelease;
YAC_String cb_args_fxcombobox_onListClicked;

// ---------------------------------------------------------------------------------------------
class TKS_FXApp : public YAC_Object {
public:
	YAC_H(TKS_FXApp);

	void yacGetConstantStringList(YAC_String *_r);

	void tksinter_create(void);
	void tksinter_run(void);
	void tksinter_runModal(void);
	void tksinter_stop(void);

#define YAC_CLASS TKS_FXApp
#define YAC_NUM_COMMANDS 5

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "create"
#define YAC_CMD_1_RTTI 0
#define YAC_CMD_1_SYM tksinter_create

#define YAC_CMD_2_NAME "run"
#define YAC_CMD_2_RTTI 0
#define YAC_CMD_2_SYM tksinter_run

#define YAC_CMD_3_NAME "runModal"
#define YAC_CMD_3_RTTI 0
#define YAC_CMD_3_SYM tksinter_runModal

#define YAC_CMD_4_NAME "stop"
#define YAC_CMD_4_RTTI 0
#define YAC_CMD_4_SYM tksinter_stop

#include <yac_createcommands.h>
};
YAC_C(TKS_FXApp, "FXApp");
	
void TKS_FXApp::tksinter_create(void) {
	application->create();
}

void TKS_FXApp::tksinter_run(void) {
	application->run();
}

void TKS_FXApp::tksinter_runModal(void) {
	application->runModal();
}

void TKS_FXApp::tksinter_stop(void) {
	application->stop();
}

void TKS_FXApp::yacGetConstantStringList(YAC_String *_r) {
	_r->visit(
"KEY_BackSpace:$FF08 "
"KEY_Tab:$FF09 "
"KEY_Linefeed:$FF0A "
"KEY_Clear:$FF0B "
"KEY_Return:$FF0D "
"KEY_Pause:$FF13 "
"KEY_Scroll_Lock:$FF14 "
"KEY_Sys_Req:$FF15 "
"KEY_Escape:$FF1B "
"KEY_Delete:$FFFF "
"KEY_Multi_key:$FF20 "
"KEY_Home:$FF50 "
"KEY_Left:$FF51 "
"KEY_Up:$FF52 "
"TEST_XXX:42 "
"KEY_Right:$FF53 "
"KEY_Down:$FF54 "
"KEY_Prior:$FF55 "
"KEY_Page_Up:$FF55 "
"KEY_Next:$FF56 "
"KEY_Page_Down:$FF56 "
"KEY_End:$FF57 "
"KEY_Begin:$FF58 "
"KEY_BackSpace:$FF08 "
"KEY_Tab:$FF09 "
"KEY_Linefeed:$FF0A "
"KEY_Clear:$FF0B "
"KEY_Return:$FF0D "
"KEY_Pause:$FF13 "
"KEY_Scroll_Lock:$FF14 "
"KEY_Sys_Req:$FF15 "
"KEY_Escape:$FF1B "
"KEY_Delete:$FFFF "
"KEY_Multi_key:$FF20 "
"KEY_Home:$FF50 "
"KEY_Left:$FF51 "
"KEY_Up:$FF52 "
"KEY_Right:$FF53 "
"KEY_Down:$FF54 "
"KEY_Prior:$FF55 "
"KEY_Page_Up:$FF55 "
"KEY_Next:$FF56 "
"KEY_Page_Down:$FF56 "
"KEY_End:$FF57 "
"KEY_Begin:$FF58 "
"KEY_Select:$FF60 "
"KEY_Print:$FF61 "
"KEY_Execute:$FF62 "
"KEY_Insert:$FF63 "
"KEY_Undo:$FF65 "
"KEY_Redo:$FF66 "
"KEY_Menu:$FF67 "
"KEY_Find:$FF68 "
"KEY_Cancel:$FF69 "
"KEY_Help:$FF6A "
"KEY_Break:$FF6B "
"KEY_Mode_switch:$FF7E "
"KEY_script_switch:$FF7E "
"KEY_Num_Lock:$FF7F "
"KEY_KP_Space:$FF80 "
"KEY_KP_Tab:$FF89 "
"KEY_KP_Enter:$FF8D "
"KEY_KP_F1:$FF91 "
"KEY_KP_F2:$FF92 "
"KEY_KP_F3:$FF93 "
"KEY_KP_F4:$FF94 "
"KEY_KP_Home:$FF95 "
"KEY_KP_Left:$FF96 "
"KEY_KP_Up:$FF97 "
"KEY_KP_Right:$FF98 "
"KEY_KP_Down:$FF99 "
"KEY_KP_Prior:$FF9A "
"KEY_KP_Page_Up:$FF9A "
"KEY_KP_Next:$FF9B "
"KEY_KP_Page_Down:$FF9B "
"KEY_KP_End:$FF9C "
"KEY_KP_Begin:$FF9D "
"KEY_KP_Insert:$FF9E "
"KEY_KP_Delete:$FF9F "
"KEY_KP_Equal:$FFBD "
"KEY_KP_Multiply:$FFAA "
"KEY_KP_Add:$FFAB "
"KEY_KP_Separator:$FFAC "
"KEY_KP_Subtract:$FFAD "
"KEY_KP_Decimal:$FFAE "
"KEY_KP_Divide:$FFAF "
"KEY_KP_0:$FFB0 "
"KEY_KP_1:$FFB1 "
"KEY_KP_2:$FFB2 "
"KEY_KP_3:$FFB3 "
"KEY_KP_4:$FFB4 "
"KEY_KP_5:$FFB5 "
"KEY_KP_6:$FFB6 "
"KEY_KP_7:$FFB7 "
"KEY_KP_8:$FFB8 "
"KEY_KP_9:$FFB9 "
"KEY_F1:$FFBE "
"KEY_F2:$FFBF "
"KEY_F3:$FFC0 "
"KEY_F4:$FFC1 "
"KEY_F5:$FFC2 "
"KEY_F6:$FFC3 "
"KEY_F7:$FFC4 "
"KEY_F8:$FFC5 "
"KEY_F9:$FFC6 "
"KEY_F10:$FFC7 "
"KEY_F11:$FFC8 "
"KEY_L1:$FFC8 "
"KEY_F12:$FFC9 "
"KEY_L2:$FFC9 "
"KEY_F13:$FFCA "
"KEY_L3:$FFCA "
"KEY_F14:$FFCB "
"KEY_L4:$FFCB "
"KEY_F15:$FFCC "
"KEY_L5:$FFCC "
"KEY_F16:$FFCD "
"KEY_L6:$FFCD "
"KEY_F17:$FFCE "
"KEY_L7:$FFCE "
"KEY_F18:$FFCF "
"KEY_L8:$FFCF "
"KEY_F19:$FFD0 "
"KEY_L9:$FFD0 "
"KEY_F20:$FFD1 "
"KEY_L10:$FFD1 "
"KEY_F21:$FFD2 "
"KEY_R1:$FFD2 "
"KEY_F22:$FFD3 "
"KEY_R2:$FFD3 "
"KEY_F23:$FFD4 "
"KEY_R3:$FFD4 "
"KEY_F24:$FFD5 "
"KEY_R4:$FFD5 "
"KEY_F25:$FFD6 "
"KEY_R5:$FFD6 "
"KEY_F26:$FFD7 "
"KEY_R6:$FFD7 "
"KEY_F27:$FFD8 "
"KEY_R7:$FFD8 "
"KEY_F28:$FFD9 "
"KEY_R8:$FFD9 "
"KEY_F29:$FFDA "
"KEY_R9:$FFDA "
"KEY_F30:$FFDB "
"KEY_R10:$FFDB "
"KEY_F31:$FFDC "
"KEY_R11:$FFDC "
"KEY_F32:$FFDD "
"KEY_R12:$FFDD "
"KEY_F33:$FFDE "
"KEY_R13:$FFDE "
"KEY_F34:$FFDF "
"KEY_R14:$FFDF "
"KEY_F35:$FFE0 "
"KEY_R15:$FFE0 "
"KEY_Shift_L:$FFE1 "
"KEY_Shift_R:$FFE2 "
"KEY_Control_L:$FFE3 "
"KEY_Control_R:$FFE4 "
"KEY_Caps_Lock:$FFE5 "
"KEY_Shift_Lock:$FFE6 "
"KEY_Meta_L:$FFE7 "
"KEY_Meta_R:$FFE8 "
"KEY_Alt_L:$FFE9 "
"KEY_Alt_R:$FFEA "
"KEY_Super_L:$FFEB "
"KEY_Super_R:$FFEC "
"KEY_Hyper_L:$FFED "
"KEY_Hyper_R:$FFEE "
"KEY_ISO_Lock:$FE01 "
"KEY_ISO_Level2_Latch:$FE02 "
"KEY_ISO_Level3_Shift:$FE03 "
"KEY_ISO_Level3_Latch:$FE04 "
"KEY_ISO_Level3_Lock:$FE05 "
"KEY_ISO_Group_Shift:$FF7E "
"KEY_ISO_Group_Latch:$FE06 "
"KEY_ISO_Group_Lock:$FE07 "
"KEY_ISO_Next_Group:$FE08 "
"KEY_ISO_Next_Group_Lock:$FE09 "
"KEY_ISO_Prev_Group:$FE0A "
"KEY_ISO_Prev_Group_Lock:$FE0B "
"KEY_ISO_First_Group:$FE0C "
"KEY_ISO_First_Group_Lock:$FE0D "
"KEY_ISO_Last_Group:$FE0E "
"KEY_ISO_Last_Group_Lock:$FE0F "
"KEY_ISO_Left_Tab:$FE20 "
"KEY_ISO_Move_Line_Up:$FE21 "
"KEY_ISO_Move_Line_Down:$FE22 "
"KEY_ISO_Partial_Line_Up:$FE23 "
"KEY_ISO_Partial_Line_Down:$FE24 "
"KEY_ISO_Partial_Space_Left:$FE25 "
"KEY_ISO_Partial_Space_Right:$FE26 "
"KEY_ISO_Set_Margin_Left:$FE27 "
"KEY_ISO_Set_Margin_Right:$FE28 "
"KEY_ISO_Release_Margin_Left:$FE29 "
"KEY_ISO_Release_Margin_Right:$FE2A "
"KEY_ISO_Release_Both_Margins:$FE2B "
"KEY_ISO_Fast_Cursor_Left:$FE2C "
"KEY_ISO_Fast_Cursor_Right:$FE2D "
"KEY_ISO_Fast_Cursor_Up:$FE2E "
"KEY_ISO_Fast_Cursor_Down:$FE2F "
"KEY_ISO_Continuous_Underline:$FE30 "
"KEY_ISO_Discontinuous_Underline:$FE31 "
"KEY_ISO_Emphasize:$FE32 "
"KEY_ISO_Center_Object:$FE33 "
"KEY_ISO_Enter:$FE34 "
"KEY_dead_grave:$FE50 "
"KEY_dead_acute:$FE51 "
"KEY_dead_circumflex:$FE52 "
"KEY_dead_tilde:$FE53 "
"KEY_dead_macron:$FE54 "
"KEY_dead_breve:$FE55 "
"KEY_dead_abovedot:$FE56 "
"KEY_dead_diaeresis:$FE57 "
"KEY_dead_abovering:$FE58 "
"KEY_dead_doubleacute:$FE59 "
"KEY_dead_caron:$FE5A "
"KEY_dead_cedilla:$FE5B "
"KEY_dead_ogonek:$FE5C "
"KEY_dead_iota:$FE5D "
"KEY_dead_voiced_sound:$FE5E "
"KEY_dead_semivoiced_sound:$FE5F "
"KEY_dead_belowdot:$FE60 "
"KEY_First_Virtual_Screen:$FED0 "
"KEY_Prev_Virtual_Screen.$FED1 "
"KEY_Next_Virtual_Screen:$FED2 "
"KEY_Last_Virtual_Screen:$FED4 "
"KEY_Terminate_Server:$FED5 "
"KEY_AccessX_Enable:$FE70 "
"KEY_AccessX_Feedback_Enable:$FE71 "
"KEY_RepeatKeys_Enable:$FE72 "
"KEY_SlowKeys_Enable:$FE73 "
"KEY_BounceKeys_Enable:$FE74 "
"KEY_StickyKeys_Enable:$FE75 "
"KEY_MouseKeys_Enable:$FE76 "
"KEY_MouseKeys_Accel_Enable:$FE77 "
"KEY_Overlay1_Enable:$FE78 "
"KEY_Overlay2_Enable:$FE79 "
"KEY_AudibleBell_Enable:$FE7A "
"KEY_Pointer_Left:$FEE0 "
"KEY_Pointer_Right:$FEE1 "
"KEY_Pointer_Up:$FEE2 "
"KEY_Pointer_Down:$FEE3 "
"KEY_Pointer_UpLeft:$FEE4 "
"KEY_Pointer_UpRight:$FEE5 "
"KEY_Pointer_DownLeft:$FEE6 "
"KEY_Pointer_DownRight:$FEE7 "
"KEY_Pointer_Button_Dflt_$FEE8 "
"KEY_Pointer_Button1:$FEE9 "
"KEY_Pointer_Button2:$FEEA "
"KEY_Pointer_Button3:$FEEB "
"KEY_Pointer_Button4:$FEEC "
"KEY_Pointer_Button5:$FEED "
"KEY_Pointer_DblClick_Dflt:$FEEE "
"KEY_Pointer_DblClick1:$FEEF "
"KEY_Pointer_DblClick2:$FEF0 "
"KEY_Pointer_DblClick3:$FEF1 "
"KEY_Pointer_DblClick4:$FEF2 "
"KEY_Pointer_DblClick5:$FEF3 "
"KEY_Pointer_Drag_Dflt:$FEF4 "
"KEY_Pointer_Drag1:$FEF5 "
"KEY_Pointer_Drag2:$FEF6 "
"KEY_Pointer_Drag3:$FEF7 "
"KEY_Pointer_Drag4:$FEF8 "
"KEY_Pointer_Drag5:$FEFD "
"KEY_Pointer_EnableKeys:$FEF9 "
"KEY_Pointer_Accelerate:$FEFA "
"KEY_Pointer_DfltBtnNext:$FEFB "
"KEY_Pointer_DfltBtnPrev:$FEFC "
"KEY_3270_Duplicate:$FD01 "
"KEY_3270_FieldMark:$FD02 "
"KEY_3270_Right2:$FD03 "
"KEY_3270_Left2:$FD04 "
"KEY_3270_BackTab:$FD05 "
"KEY_3270_EraseEOF:$FD06 "
"KEY_3270_EraseInput:$FD07 "
"KEY_3270_Reset:$FD08 "
"KEY_3270_Quit:$FD09 "
"KEY_3270_PA1:$FD0A "
"KEY_3270_PA2:$FD0B "
"KEY_3270_PA3:$FD0C "
"KEY_3270_Test:$FD0D "
"KEY_3270_Attn:$FD0E "
"KEY_3270_CursorBlink:$FD0F "
"KEY_3270_AltCursor:$FD10 "
"KEY_3270_KeyClick:$FD11 "
"KEY_3270_Jump:$FD12 "
"KEY_3270_Ident:$FD13 "
"KEY_3270_Rule:$FD14 "
"KEY_3270_Copy:$FD15 "
"KEY_3270_Play:$FD16 "
"KEY_3270_Setup:$FD17 "
"KEY_3270_Record:$FD18 "
"KEY_3270_ChangeScreen:$FD19 "
"KEY_3270_DeleteWord:$FD1A "
"KEY_3270_ExSelect:$FD1B "
"KEY_3270_CursorSelect:$FD1C "
"KEY_3270_PrintScreen:$FD1D "
"KEY_3270_Enter:$FD1E "
"KEY_space:$20 "
"KEY_exclam:$21 "
"KEY_quotedbl:$22 "
"KEY_numbersign:$23 "
"KEY_dollar:$24 "
"KEY_percent:$25 "
"KEY_ampersand:$26 "
"KEY_apostrophe:$27 "
"KEY_quoteright:$27 "
"KEY_parenleft:$28 "
"KEY_parenright:$29 "
"KEY_asterisk:$2A "
"KEY_plus:$2B "
"KEY_comma:$2C "
"KEY_minus:$2D "
"KEY_period:$2E "
"KEY_slash:$2F "
"KEY_0:$30 "
"KEY_1:$31 "
"KEY_2:$32 "
"KEY_3:$33 "
"KEY_4:$34 "
"KEY_5:$35 "
"KEY_6:$36 "
"KEY_7:$37 "
"KEY_8:$38 "
"KEY_9:$39 "
"KEY_colon:$3A "
"KEY_semicolon:$3B "
"KEY_less:$3C "
"KEY_equal:$3D "
"KEY_greater:$3E "
"KEY_question:$3F "
"KEY_at:$40 "
"KEY_A:$41 "
"KEY_B:$42 "
"KEY_C:$43 "
"KEY_D:$44 "
"KEY_E:$45 "
"KEY_F:$46 "
"KEY_G:$47 "
"KEY_H:$48 "
"KEY_I:$49 "
"KEY_J:$4A "
"KEY_K:$4B "
"KEY_L:$4C "
"KEY_M:$4D "
"KEY_N:$4E "
"KEY_O:$4F "
"KEY_P:$50 "
"KEY_Q:$51 "
"KEY_R:$52 "
"KEY_S:$53 "
"KEY_T:$54 "
"KEY_U:$55 "
"KEY_V:$56 "
"KEY_W:$57 "
"KEY_X:$58 "
"KEY_Y:$59 "
"KEY_Z:$5A "
"KEY_bracketleft:$5B "
"KEY_backslash:$5C "
"KEY_bracketright:$5D "
"KEY_asciicircum:$5E "
"KEY_underscore:$5F "
"KEY_grave:$60 "
"KEY_quoteleft:$60 "
"KEY_a:$61 "
"KEY_b:$62 "
"KEY_c:$63 "
"KEY_d:$64 "
"KEY_e:$65 "
"KEY_f:$66 "
"KEY_g:$67 "
"KEY_h:$68 "
"KEY_i:$69 "
"KEY_j:$6A "
"KEY_k:$6B "
"KEY_l:$6C "
"KEY_m:$6D "
"KEY_n:$6E "
"KEY_o:$6F "
"KEY_p:$70 "
"KEY_q:$71 "
"KEY_r:$72 "
"KEY_s:$73 "
"KEY_t:$74 "
"KEY_u:$75 "
"KEY_v:$76 "
"KEY_w:$77 "
"KEY_x:$78 "
"KEY_y:$79 "
"KEY_z:$7A "
"KEY_braceleft:$7B "
"KEY_bar:$7C "
"KEY_braceright:$7D "
"KEY_asciitilde:$7E "
"KEY_nobreakspace:$A0 "
"KEY_exclamdown:$A1 "
"KEY_cent:$A2 "
"KEY_sterling:$A3 "
"KEY_currency:$A4 "
"KEY_yen:$A5 "
"KEY_brokenbar:$A6 "
"KEY_section:$A7 "
"KEY_diaeresis:$A8 "
"KEY_copyright:$A9 "
"KEY_ordfeminine:$AA "
"KEY_guillemotleft:$AB "
"KEY_notsign:$AC "
"KEY_hyphen:$AD "
"KEY_registered:$AE "
"KEY_macron:$AF "
"KEY_degree:$B0 "
"KEY_plusminus:$B1 "
"KEY_twosuperior:$B2 "
"KEY_threesuperior:$B3 "
"KEY_acute:$B4 "
"KEY_mu:$B5 "
"KEY_paragraph:$B6 "
"KEY_periodcentered:$B7 "
"KEY_cedilla:$B8 "
"KEY_onesuperior:$B9 "
"KEY_masculine:$BA "
"KEY_guillemotright:$BB "
"KEY_onequarter:$BC "
"KEY_onehalf:$BD "
"KEY_threequarters:$BE "
"KEY_questiondown:$BF "
"KEY_Agrave:$C0 "
"KEY_Aacute:$C1 "
"KEY_Acircumflex:$C2 "
"KEY_Atilde:$C3 "
"KEY_Adiaeresis:$C4 "
"KEY_Aring:$C5 "
"KEY_AE:$C6 "
"KEY_Ccedilla:$C7 "
"KEY_Egrave:$C8 "
"KEY_Eacute:$C9 "
"KEY_Ecircumflex:$CA "
"KEY_Ediaeresis:$CB "
"KEY_Igrave:$CC "
"KEY_Iacute:$CD "
"KEY_Icircumflex:$CE "
"KEY_Idiaeresis:$CF "
"KEY_ETH:$D0 "
"KEY_Eth:$D0 "
"KEY_Ntilde:$D1 "
"KEY_Ograve:$D2 "
"KEY_Oacute:$D3 "
"KEY_Ocircumflex:$D4 "
"KEY_Otilde:$D5 "
"KEY_Odiaeresis:$D6 "
"KEY_multiply:$D7 "
"KEY_Ooblique:$D8 "
"KEY_Ugrave:$D9 "
"KEY_Uacute:$DA "
"KEY_Ucircumflex:$DB "
"KEY_Udiaeresis:$DC "
"KEY_Yacute:$DD "
"KEY_THORN:$DE "
"KEY_Thorn:$DE "
"KEY_ssharp:$DF "
"KEY_agrave:$E0 "
"KEY_aacute:$E1 "
"KEY_acircumflex:$E2 "
"KEY_atilde:$E3 "
"KEY_adiaeresis:$E4 "
"KEY_aring:$E5 "
"KEY_ae:$E6 "
"KEY_ccedilla:$E7 "
"KEY_egrave:$E8 "
"KEY_eacute:$E9 "
"KEY_ecircumflex:$EA "
"KEY_ediaeresis:$EB "
"KEY_igrave:$EC "
"KEY_iacute:$ED "
"KEY_icircumflex:$EE "
"KEY_idiaeresis:$EF "
"KEY_eth:$F0 "
"KEY_ntilde:$F1 "
"KEY_ograve:$F2 "
"KEY_oacute:$F3 "
"KEY_ocircumflex:$F4 "
"KEY_otilde:$F5 "
"KEY_odiaeresis:$F6 "
"KEY_division:$F7 "
"KEY_oslash:$F8 "
"KEY_ugrave:$F9 "
"KEY_uacute:$FA "
"KEY_ucircumflex:$FB "
"KEY_udiaeresis:$FC "
"KEY_yacute:$FD "
"KEY_thorn:$FE "
"KEY_ydiaeresis:$FF "
"KEY_Select:$FF60 "
"KEY_Print:$FF61 "
"KEY_Execute:$FF62 "
"KEY_Insert:$FF63 "
"KEY_Undo:$FF65 "
"KEY_Redo:$FF66 "
"KEY_Menu:$FF67 "
"KEY_Find:$FF68 "
"KEY_Cancel:$FF69 "
"KEY_Help:$FF6A "
"KEY_Break:$FF6B "
"KEY_Mode_switch:$FF7E "
"KEY_script_switch:$FF7E "
"KEY_Num_Lock:$FF7F "
"IMAGE_KEEP:$1 "      /// Keep pixel data in client
"IMAGE_OWNED:$2 "     /// Pixel data is owned by image
"IMAGE_DITHER:$0 "    /// Dither image to look better
"IMAGE_NEAREST:$4 "   /// Turn off dithering and map to nearest color
"IMAGE_ALPHA:$8 "     /// Data has alpha channel
"IMAGE_OPAQUE:$10 "   /// Force opaque background
"IMAGE_ALPHACOLOR:$20 " /// Override transparancy color
"IMAGE_SHMI:$40 "     /// Using shared memory image
"IMAGE_SHMP:$80 "     /// Using shared memory pixmap
"IMAGE_ALPHAGUESS:$100 " /// Guess transparency color from corners
"FONTWEIGHT_DONTCARE:$0 "  /// Don't care about weight
"FONTWEIGHT_THIN:$100 "   /// Thin
"FONTWEIGHT_EXTRALIGHT:$200 "  /// Extra light
"FONTWEIGHT_LIGHT:$300 "      /// Light
"FONTWEIGHT_NORMAL:$400 "     /// Normal or regular weight
"FONTWEIGHT_REGULAR:$400 "    /// Normal or regular weight
"FONTWEIGHT_MEDIUM:$500 "     /// Medium bold face
"FONTWEIGHT_DEMIBOLD:$600 "   /// Demi bold face
"FONTWEIGHT_BOLD:$700 "       /// Bold face
"FONTWEIGHT_EXTRABOLD:$800 "  /// Extra
"FONTWEIGHT_HEAVY:$900 "      /// Heavy
"FONTWEIGHT_BLACK:$900 "      /// Black
"FONTSLANT_DONTCARE:$0 "  /// Don't care about slant
"FONTSLANT_REGULAR:$1 "   /// Regular straight up
"FONTSLANT_ITALIC:$2 "    /// Italics
"FONTSLANT_OBLIQUE:$3 "   /// Oblique slant
"FONTSLANT_REVERSE_ITALIC:$4 "  /// Reversed italic
"FONTSLANT_REVERSE_OBLIQUE:$5 " /// Reversed oblique
"FONTENCODING_DEFAULT:0 "      /// Don't care character encoding
"FONTENCODING_ISO_8859_1:1 "
"FONTENCODING_ISO_8859_2:2 "
"FONTENCODING_ISO_8859_3:3 "
"FONTENCODING_ISO_8859_4:4 "
"FONTENCODING_ISO_8859_5:5 "  /// Cyrillic (almost obsolete)
"FONTENCODING_ISO_8859_6:6 " 
"FONTENCODING_ISO_8859_7:7 "
"FONTENCODING_ISO_8859_8:8 "
"FONTENCODING_ISO_8859_9:9 "
"FONTENCODING_ISO_8859_10:10 "
"FONTENCODING_ISO_8859_11:11 "
"FONTENCODING_ISO_8859_13:13 "
"FONTENCODING_ISO_8859_14:14 "
"FONTENCODING_ISO_8859_15:15 "
"FONTENCODING_ISO_8859_16:16 "
"FONTENCODING_KOI8:17 "
"FONTENCODING_KOI8_R:18 "    /// Russian
"FONTENCODING_KOI8_U:19 "    /// Ukrainian
"FONTENCODING_KOI8_UNIFIED:20 "
"FONTENCODING_LATIN1:1 "   /// Latin 1 (West European)
"FONTENCODING_LATIN2:2 "   /// Latin 2 (East European)
"FONTENCODING_LATIN3:3 "   /// Latin 3 (South European)
"FONTENCODING_LATIN4:4 "   /// Latin 4 (North European)
"FONTENCODING_LATIN5:9 "   /// Latin 5 (Turkish)
"FONTENCODING_LATIN6:10 "  /// Latin 6 (Nordic)
"FONTENCODING_LATIN7:13 "  /// Latin 7 (Baltic Rim)
"FONTENCODING_LATIN8:14 "  /// Latin 8 (Celtic)
"FONTENCODING_LATIN9:15 "  /// Latin 9 AKA Latin 0
"FONTENCODING_LATIN10:16 " /// Latin 10
"FONTENCODING_USASCII:1 "     /// Latin 1
"FONTENCODING_WESTEUROPE:1 "  /// Latin 1 (West European)
"FONTENCODING_EASTEUROPE:2 "  /// Latin 2 (East European)
"FONTENCODING_SOUTHEUROPE:3 " /// Latin 3 (South European)
"FONTENCODING_NORTHEUROPE:4 " /// Latin 4 (North European)
"FONTENCODING_CYRILLIC:5 "    /// Cyrillic
"FONTENCODING_RUSSIAN:17 "    /// Cyrillic
"FONTENCODING_ARABIC:6 "      /// Arabic
"FONTENCODING_GREEK:7 "       /// Greek
"FONTENCODING_HEBREW:8 "      /// Hebrew
"FONTENCODING_TURKISH:9 "     /// Latin 5 (Turkish)
"FONTENCODING_NORDIC:10 "     /// Latin 6 (Nordic)
"FONTENCODING_THAI:11 "       /// Thai
"FONTENCODING_BALTIC:13 "     /// Latin 7 (Baltic Rim)
"FONTENCODING_CELTIC:14 "     /// Latin 8 (Celtic)
"FONTSETWIDTH_DONTCARE:0 "         /// Don't care about set width
"FONTSETWIDTH_ULTRACONDENSED:10 "  /// Ultra condensed printing
"FONTSETWIDTH_EXTRACONDENSED:20 "  /// Extra condensed
"FONTSETWIDTH_CONDENSED:30 "       /// Condensed
"FONTSETWIDTH_NARROW:30 "          /// Narrow
"FONTSETWIDTH_COMPRESSED:30 "      /// Compressed
"FONTSETWIDTH_SEMICONDENSED:40 "   /// Semi-condensed
"FONTSETWIDTH_MEDIUM:50 "          /// Medium printing
"FONTSETWIDTH_NORMAL:50 "          /// Normal printing
"FONTSETWIDTH_REGULAR:50 "         /// Regulat printing
"FONTSETWIDTH_SEMIEXPANDED:60 "    /// Semi expanded
"FONTSETWIDTH_EXPANDED:70 "        /// Expanded
"FONTSETWIDTH_WIDE:80 "            /// Wide
"FONTSETWIDTH_EXTRAEXPANDED:80 "   /// Extra expanded
"FONTSETWIDTH_ULTRAEXPANDED:90 "   /// Ultra expanded
"FONTPITCH_DEFAULT:0 "       /// Default pitch
"FONTPITCH_FIXED:1 "         /// Fixed pitch, mono-spaced
"FONTPITCH_VARIABLE:2 "      /// Variable pitch, proportional spacing
"FONTHINT_DONTCARE:0 "       /// Don't care which font
"FONTHINT_DECORATIVE:4 "     /// Fancy fonts
"FONTHINT_MODERN:8 "         /// Monospace typewriter font
"FONTHINT_ROMAN:16 "         /// Variable width times-like font, serif
"FONTHINT_SCRIPT:32 "        /// Script or cursive
"FONTHINT_SWISS:64 "         /// Helvetica/swiss type font, sans-serif
"FONTHINT_SYSTEM:128 "       /// System font
"FONTHINT_X11:256 "          /// X11 Font string
"FONTHINT_SCALABLE:512 "     /// Scalable fonts
"FONTHINT_POLYMORPHIC:1024 " /// Polymorphic fonts
"LAYOUT_NORMAL:0 "           /// Default layout mode
"LAYOUT_SIDE_TOP:0 "         /// Pack on top side (default)
"LAYOUT_SIDE_BOTTOM:$1 "     /// Pack on bottom side
"LAYOUT_SIDE_LEFT:$2 "       /// Pack on left side
"LAYOUT_SIDE_RIGHT:$3 "      /// Pack on right side
"LAYOUT_FILL_COLUMN:$1 "     /// Matrix column is stretchable
"LAYOUT_FILL_ROW:$2 "        /// Matrix row is stretchable
"LAYOUT_LEFT:0 "             /// Stick on left (default)
"LAYOUT_RIGHT:$4 "           /// Stick on right
"LAYOUT_CENTER_X:$8 "        /// Center horizontally
"LAYOUT_FIX_X:$c "           /// X fixed
"LAYOUT_TOP:0 "              /// Stick on top (default)
"LAYOUT_BOTTOM:$10 "         /// Stick on bottom
"LAYOUT_CENTER_Y:$20 "       /// Center vertically
"LAYOUT_FIX_Y:$30 "          /// Y fixed
"LAYOUT_RESERVED_1:$40 "
"LAYOUT_RESERVED_2:$80 "
"LAYOUT_FIX_WIDTH:$100 "     /// Width fixed
"LAYOUT_FIX_HEIGHT:$200 "    /// height fixed
"LAYOUT_MIN_WIDTH:0 "        /// Minimum width is the default
"LAYOUT_MIN_HEIGHT:0 "       /// Minimum height is the default
"LAYOUT_FILL_X:$400 "        /// Stretch or shrink horizontally
"LAYOUT_FILL_Y:$800 "        /// Stretch or shrink vertically
"LAYOUT_EXPLICIT:$33c "      /// LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT
"FRAME_NONE:0 "              /// Default is no frame
"FRAME_SUNKEN:$1000 "        /// Sunken border
"FRAME_RAISED:$2000 "        /// Raised border
"FRAME_THICK:$4000 "         /// Thick border
"FRAME_GROOVE:$4000 "        /// A groove or etched-in border
"FRAME_RIDGE:$7000 "         /// A ridge or embossed border
"FRAME_LINE:$3000 "          /// Simple line border
"FRAME_NORMAL:$5000 "        /// Regular raised/thick border
"PACK_NORMAL:0 "             /// Default is each its own size
"PACK_UNIFORM_HEIGHT:$8000 " /// Uniform height
"PACK_UNIFORM_WIDTH:$10000 " /// Uniform width
"DECOR_NONE:0 "              /// Borderless window
"DECOR_TITLE:$20000 "        /// Window title
"DECOR_MINIMIZE:$40000 "     /// Minimize button
"DECOR_MAXIMIZE:$80000 "     /// Maximize button
"DECOR_CLOSE:$100000 "       /// Close button
"DECOR_BORDER:$200000 "      /// Border
"DECOR_RESIZE:$400000 "      /// Resize handles
"DECOR_MENU:$800000 "        /// Window menu
"DECOR_ALL:$fe0000 "         /// (DECOR_TITLE|DECOR_MINIMIZE|DECOR_MAXIMIZE|DECOR_CLOSE|DECOR_BORDER|DECOR_RESIZE|DECOR_MENU)
"PLACEMENT_DEFAULT:0 "       /// Place it at the default size and location
"PLACEMENT_VISIBLE:1 "       /// Place window to be fully visible
"PLACEMENT_CURSOR:2 "        /// Place it under the cursor position
"PLACEMENT_OWNER:3 "         /// Place it centered on its owner
"PLACEMENT_SCREEN:4 "        /// Place it centered on the screen
"PLACEMENT_MAXIMIZED:5 "     /// Place it maximized to the screen size
"SCROLLERS_NORMAL:0 "        /// Show the scrollbars when needed
"HSCROLLER_ALWAYS:$8000 "    /// Always show horizontal scrollers
"HSCROLLER_NEVER:$10000 "    /// Never show horizontal scrollers
"VSCROLLER_ALWAYS:$20000 "   /// Always show vertical scrollers
"VSCROLLER_NEVER:$40000 "    /// Never show vertical scrollers
"HSCROLLING_ON:0 "           /// Horizontal scrolling turned on (default)
"HSCROLLING_OFF:$18000 "     /// Horizontal scrolling turned off
"VSCROLLING_ON:0 "           /// Vertical scrolling turned on (default)
"VSCROLLING_OFF:$60000 "     /// Vertical scrolling turned off
"SCROLLERS_TRACK:0 "         /// Scrollers track continuously for smooth scrolling
"SCROLLERS_DONT_TRACK:$80000 " /// Scrollers don't track continuously
"JUSTIFY_NORMAL:0 "          /// Default justification is centered text
"JUSTIFY_CENTER_X:0 "        /// Text is centered horizontally
"JUSTIFY_LEFT:$8000 "        /// Text is left-justified
"JUSTIFY_RIGHT:$10000 "      /// Text is right-justified
"JUSTIFY_HZ_APART:$18000 "   /// Combination of JUSTIFY_LEFT & JUSTIFY_RIGHT
"JUSTIFY_CENTER_Y:0 "        /// Text is centered vertically
"JUSTIFY_TOP:$20000 "        /// Text is aligned with label top
"JUSTIFY_BOTTOM:$40000 "     /// Text is aligned with label bottom
"JUSTIFY_VT_APART:$60000 "   /// Combination of JUSTIFY_TOP & JUSTIFY_BOTTOM
"ICON_UNDER_TEXT:0 "        /// Icon appears under text
"ICON_AFTER_TEXT:$80000 "   /// Icon appears after text (to its right)
"ICON_BEFORE_TEXT:$100000 " /// Icon appears before text (to its left)
"ICON_ABOVE_TEXT:$200000 "  /// Icon appears above text
"ICON_BELOW_TEXT:$400000 "  /// Icon appears below text
"TEXT_OVER_ICON:0 "         /// Same as ICON_UNDER_TEXT
"TEXT_AFTER_ICON:$100000 "  /// Same as ICON_BEFORE_TEXT
"TEXT_BEFORE_ICON:$80000 "  /// Same as ICON_AFTER_TEXT
"TEXT_ABOVE_ICON:$400000 "  /// Same as ICON_BELOW_TEXT
"TEXT_BELOW_ICON:$200000 "  /// Same as ICON_ABOVE_TEXT
"LABEL_NORMAL:$100000 "     ///  JUSTIFY_NORMAL|ICON_BEFORE_TEXT
"BUTTON_AUTOGRAY:$800000 "  /// Automatically gray out when not updated
"BUTTON_AUTOHIDE:$1000000 " /// Automatically hide button when not updated
"BUTTON_TOOLBAR:$2000000 "  /// Toolbar style button [flat look]
"BUTTON_DEFAULT:$4000000 "  /// May become default button when receiving focus
"BUTTON_INITIAL:$8000000 "  /// This button is the initial default button
"BUTTON_NORMAL :$106000 "   /// (FRAME_RAISED|FRAME_THICK|JUSTIFY_NORMAL|ICON_BEFORE_TEXT)
"SCROLLBAR_HORIZONTAL:$20000 "  /// Horizontally oriented
"SCROLLBAR_VERTICAL:0 "         /// Vertically oriented
"SLIDER_HORIZONTAL:0 "          /// Slider shown horizontally
"SLIDER_VERTICAL:$8000 "        /// Slider shown vertically
"SLIDER_ARROW_UP:$10000 "       /// Slider has arrow head pointing up
"SLIDER_ARROW_DOWN:$20000 "     /// Slider has arrow head pointing down
"SLIDER_ARROW_LEFT:$10000 "     /// Slider has arrow head pointing left
"SLIDER_ARROW_RIGHT:$20000 "    /// Slider has arrow head pointing right
"SLIDER_INSIDE_BAR:$40000 "     /// Slider is inside the slot rather than overhanging
"SLIDER_TICKS_TOP:$80000 "      /// Ticks on the top of horizontal slider
"SLIDER_TICKS_BOTTOM:$100000 "  /// Ticks on the bottom of horizontal slider
"SLIDER_TICKS_LEFT:$80000 "     /// Ticks on the left of vertical slider
"SLIDER_TICKS_RIGHT:$100000 "   /// Ticks on the right of vertical slider
"SLIDER_NORMAL:0 "
"TEXTFIELD_PASSWD:$800000 "       /// Password mode
"TEXTFIELD_INTEGER:$1000000 "     /// Integer mode
"TEXTFIELD_REAL:$2000000 "        /// Real mode
"TEXTFIELD_READONLY:$4000000 "    /// NOT editable
"TEXTFIELD_ENTER_ONLY:$8000000 "  /// Only callback when enter hit
"TEXTFIELD_LIMITED:$10000000 "    /// Limit entry to given number of columns
"TEXTFIELD_OVERSTRIKE:$20000000 " /// Overstrike mode
"TEXTFIELD_NORMAL:$5000 "         /// FRAME_SUNKEN|FRAME_THICK
"DIAL_VERTICAL:0 "                // Vertically oriented
"DIAL_HORIZONTAL:$8000 "          // Horizontal oriented
"DIAL_CYCLIC:$10000 "             // Value wraps around
"DIAL_HAS_NOTCH:$20000 "          // Dial has a Center Notch
"DIAL_NORMAL:0 "
"GROUPBOX_TITLE_LEFT:0 "          /// Title is left-justified
"GROUPBOX_TITLE_CENTER:$20000 "   /// Title is centered
"GROUPBOX_TITLE_RIGHT:$40000 "    /// Title is right-justified
"GROUPBOX_NORMAL:0 "
"SEPARATOR_NONE:0 "               /// Nothing visible
"SEPARATOR_GROOVE:$8000 "         /// Etched-in looking groove
"SEPARATOR_RIDGE:$10000 "         /// Embossed looking ridge
"SEPARATOR_LINE:$20000  "         /// Simple line
"MBOX_OK:$10000000 "               /// Message box has a only an OK button
"MBOX_OK_CANCEL:$20000000 "        /// Message box has OK and CANCEL buttons
"MBOX_YES_NO:$30000000 "           /// Message box has YES and NO buttons
"MBOX_YES_NO_CANCEL:$40000000 "    /// Message box has YES, NO, and CANCEL buttons
"MBOX_QUIT_CANCEL:$50000000 "      /// Message box has QUIT and CANCEL buttons
"MBOX_QUIT_SAVE_CANCEL:$60000000 " /// Message box has QUIT, SAVE, and CANCEL buttons
"MBOX_CLICKED_YES:1 "             /// The YES button was clicked
"MBOX_CLICKED_NO:2 "              /// The NO button was clicked
"MBOX_CLICKED_OK:3 "              /// The OK button was clicked
"MBOX_CLICKED_CANCEL:4 "          /// The CANCEL button was clicked
"MBOX_CLICKED_QUIT:5 "            /// The QUIT button was clicked
"MBOX_CLICKED_SAVE:6 "            /// The SAVE button was clicked
"FILELIST_SHOWHIDDEN:$4000000 "   /// Show hidden files or directories
"FILELIST_SHOWDIRS:$8000000 "     /// Show only directories
"FILELIST_NO_OWN_ASSOC:$10000000 "  /// Do not create associations for files
"SELECTFILE_ANY:0 "              /// A single file, existing or not (to save to)
"SELECTFILE_EXISTING:1 "         /// An existing file (to load)
"SELECTFILE_MULTIPLE:2 "         /// Multiple existing files
"SELECTFILE_MULTIPLE_ALL:3 "     /// Multiple existing files or directories
"SELECTFILE_DIRECTORY:4 "        /// Existing directory
"FOURSPLITTER_TRACKING:$8000 "   /// Track continuously during split
"FOURSPLITTER_NORMAL:0 "
"CHECKBUTTON_AUTOGRAY:$800000 "    /// Automatically gray out when not updated
"CHECKBUTTON_AUTOHIDE:$1000000 "   /// Automatically hide when not updated
"CHECKBUTTON_NORMAL:$100000 "      /// JUSTIFY_NORMAL|ICON_BEFORE_TEXT
"COMBOBOX_NO_REPLACE:0 "           /// Leave the list the same
"COMBOBOX_REPLACE:$20000 "         /// Replace current item with typed text
"COMBOBOX_INSERT_BEFORE:$40000 "   /// Typed text inserted before current
"COMBOBOX_INSERT_AFTER:$60000 "    /// Typed text inserted after current
"COMBOBOX_INSERT_FIRST:$80000 "    /// Typed text inserted at begin of list
"COMBOBOX_INSERT_LAST:$90000 "     /// Typed text inserted at end of list
"COMBOBOX_STATIC:$100000 "         /// Unchangable text box
"COMBOBOX_NORMAL:0 "               /// Can type text but list is not changed
);
}

// ---------------------------------------------------------------------------------------------
class TKS_FXMainWindow : public FXMainWindow/*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXMainWindow)

public:

	long onPaint(FXObject*,FXSelector,void*);

	TKS_FXMainWindow(void);
	TKS_FXMainWindow(sUI _opts);
	~TKS_FXMainWindow() { }

	void create(void);

public:
	YAC_H(TKS_FXMainWindow);

/*	void tksinter_setTitle(YAC_Object*);
	void tksinter_getTitle(YAC_Value*);
	void tksinter_setWidth(sSI);
	void tksinter_setHeight(sSI);
	void tksinter_getWidth(YAC_Value*);
	void tksinter_getHeight(YAC_Value*);*/

#define YAC_CLASS TKS_FXMainWindow
#define YAC_NUM_COMMANDS 7

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setTitle"
#define YAC_CMD_1_RTTI 3
#define YAC_CMD_1_SYM tksinter_setTitle

#define YAC_CMD_2_NAME "setWidth"
#define YAC_CMD_2_RTTI 1
#define YAC_CMD_2_SYM tksinter_setWidth

#define YAC_CMD_3_NAME "setHeight"
#define YAC_CMD_3_RTTI 1
#define YAC_CMD_3_SYM tksinter_setHeight

#define YAC_CMD_4_NAME "getWidth"
#define YAC_CMD_4_RTTI 0
#define YAC_CMD_4_RET 1
#define YAC_CMD_4_SYM tksinter_getWidth

#define YAC_CMD_5_NAME "getHeight"
#define YAC_CMD_5_RTTI 0
#define YAC_CMD_5_RET 1
#define YAC_CMD_5_SYM tksinter_getHeight

#define YAC_CMD_6_NAME "getTitle"
#define YAC_CMD_6_RTTI 0
#define YAC_CMD_6_RET 4
#define YAC_CMD_6_SYM tksinter_getTitle

#include <yac_createcommands.h>
};

YAC_C(TKS_FXMainWindow, "FXMainWindow");

TKS_FXMainWindow::TKS_FXMainWindow(void) /*: FXMainWindow(application,"",NULL,NULL,0)*/ {
}

TKS_FXMainWindow::TKS_FXMainWindow(sUI _opts) : FXMainWindow(application,"",NULL,NULL,_opts) {
}

void FXTopWindow::tksinter_setTitle(YAC_Object*_s) {
	if(YAC_CHK(_s, YAC_CLID_STRING))
	{
		YAC_String*s=(YAC_String*)_s;
		FXTopWindow::setTitle(s->chars);
	}
}

void FXTopWindow::tksinter_getTitle(YAC_Value *_r) {
	YAC_String*s=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
	FXString fxs=FXTopWindow::getTitle(); 
	s->visit((char*)fxs.text());
	YAC_RETS(s,1);
}

void FXWindow::tksinter_setWidth(sSI _i) {
	FXWindow::setWidth(_i);
}

void FXWindow::tksinter_setHeight(sSI _i) {
	FXWindow::setHeight(_i);
}

void FXWindow::tksinter_getWidth(YAC_Value *_r) {
	YAC_RETI(FXWindow::getWidth());
}

void FXWindow::tksinter_getHeight(YAC_Value *_r) {
	YAC_RETI(FXWindow::getHeight());
}


// Message Map for the Scribble Window class
FXDEFMAP(TKS_FXMainWindow) TKS_FXMainWindowMap[]={

  //________Message_Type_____________________ID____________Message_Handler_______
  FXMAPFUNC(SEL_PAINT,             0, TKS_FXMainWindow::onPaint),
  //FXMAPFUNC(SEL_LEFTBUTTONPRESS,   0, TKS_FXMainWindow::onMouseDown),
  //FXMAPFUNC(SEL_LEFTBUTTONRELEASE, 0, TKS_FXMainWindow::onMouseUp),
  //FXMAPFUNC(SEL_MOTION,            0, TKS_FXMainWindow::onMouseMove)
};

long TKS_FXMainWindow::onPaint(FXObject*_o,FXSelector _s, void*_e) {
	return FXMainWindow::onPaint(_o, _s, _e);
}

void TKS_FXMainWindow::create(void) {
	FXMainWindow::create();
	show(PLACEMENT_SCREEN);
}

// Macro for the ScribbleApp class hierarchy implementation
FXIMPLEMENT(TKS_FXMainWindow,FXMainWindow,TKS_FXMainWindowMap,ARRAYNUMBER(TKS_FXMainWindowMap))

// ---------------------------------------------------------------------------------------------
class TKS_FXLabel : public FXLabel/*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXLabel)

public:	
	TKS_FXLabel(void) { }

	TKS_FXLabel(FXComposite *, sUI _opts);

	~TKS_FXLabel() {}

public:
	YAC_H(TKS_FXLabel);

	//void tksinter_setText(YAC_Object *);
	//void tksinter_getText(YAC_Value*_r);

#define YAC_CLASS TKS_FXLabel
#define YAC_NUM_COMMANDS 3

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setText"
#define YAC_CMD_1_RTTI 3
#define YAC_CMD_1_SYM tksinter_setText

#define YAC_CMD_2_NAME "getText"
#define YAC_CMD_2_RTTI 0
#define YAC_CMD_2_RET 4
#define YAC_CMD_2_SYM tksinter_getText

#include <yac_createcommands.h>
};

YAC_C(TKS_FXLabel, "FXLabel");

void FXLabel::tksinter_setText(YAC_Object *_s) {
	if(YAC_CHK(_s, YAC_CLID_STRING))
	{
		YAC_String *s=(YAC_String*)_s;
		FXLabel::setText(s->chars);
	}
}

void FXLabel::tksinter_getText(YAC_Value *_r) {
	YAC_String *s=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
	FXString fxs=getText();
	s->copy((char*)fxs.text());
	YAC_RETS(s,1);
}

FXIMPLEMENT(TKS_FXLabel,FXLabel,0,0)

TKS_FXLabel::TKS_FXLabel(FXComposite *_p, sUI _opts) : FXLabel(_p, 0, 0, _opts) {
}

// ---------------------------------------------------------------------------------------------
class TKS_FXButton : public FXButton/*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXButton)

public:	
	TKS_FXButton(void) { }

	TKS_FXButton(FXComposite *, sUI _opts);

	~TKS_FXButton() {}

	//long onPaint(FXObject*,FXSelector,void*);
	long onUpdate(FXObject*,FXSelector,void*);
	long onEnter(FXObject*,FXSelector,void*);
	long onLeave(FXObject*,FXSelector,void*);
	long onFocusIn(FXObject*,FXSelector,void*);
	long onFocusOut(FXObject*,FXSelector,void*);
	long onUngrabbed(FXObject*,FXSelector,void*);
	long onLeftBtnPress(FXObject*,FXSelector,void*);
	long onLeftBtnRelease(FXObject*,FXSelector,void*);
	long onKeyPress(FXObject*,FXSelector,void*);
	long onKeyRelease(FXObject*,FXSelector,void*);
	long onHotKeyPress(FXObject*,FXSelector,void*);
	long onHotKeyRelease(FXObject*,FXSelector,void*);
	long onCheck(FXObject*,FXSelector,void*);
	long onUncheck(FXObject*,FXSelector,void*);

		/*
	enum __ids {
		ID_UPDATE=FXButton::ID_LAST,
		ID_ENTER,
		ID_LEAVE,
		ID_FOCUSIN,
		ID_FOCUSOUT,
		ID_UNGRABBED,
		ID_LEFTBTNPRESS,
		ID_LEFTBTNRELEASE,
		ID_KEYPRESS,
		ID_KEYRELEASE,

		ID_SCRIPT=FXButton::ID_LAST,

		ID_LAST
	};
		*/

public:
	void *sc_onUpdate;
	void *sc_onEnter;
	void *sc_onLeave;
	void *sc_onFocusIn;
	void *sc_onFocusOut;
	void *sc_onUngrabbed;
	void *sc_onLeftBtnPress;
	void *sc_onLeftBtnRelease;
	void *sc_onKeyPress;
	void *sc_onKeyRelease;
	void *sc_onHotKeyPress;
	void *sc_onHotKeyRelease;
	void *sc_onCheck;
	void *sc_onUncheck;

public:
	YAC_H(TKS_FXButton);

	void tksinter_setText(YAC_Object *);
	void tksinter_getText(YAC_Value*_r);

#define YAC_CLASS TKS_FXButton
#define YAC_NUM_COMMANDS 3

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setText"
#define YAC_CMD_1_RTTI 3
#define YAC_CMD_1_SYM tksinter_setText

#define YAC_CMD_2_NAME "getText"
#define YAC_CMD_2_RTTI 0
#define YAC_CMD_2_RET 4
#define YAC_CMD_2_SYM tksinter_getText

#include <yac_createcommands.h>
};

YAC_C(TKS_FXButton, "FXButton");

void TKS_FXButton::tksinter_setText(YAC_Object *_s) {
	if(YAC_CHK(_s, YAC_CLID_STRING))
	{
		YAC_String *s=(YAC_String*)_s;
		//yac_host->printf("tksinter_setText(\"%s\") this=%08x\n", s->chars,(sUI)(FXButton*)this);
		FXButton::setText(s->chars);
	}
}

void TKS_FXButton::tksinter_getText(YAC_Value *_r) {
	YAC_String *s=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
	FXString fxs=getText();
	
	s->copy((char*)fxs.text());
	YAC_RETS(s,1);
}

FXDEFMAP(TKS_FXButton) TKS_FXButtonMap[]={
  //FXMAPFUNC(SEL_PAINT,0,FXButton::onPaint),
  /*
  FXMAPFUNC(SEL_UPDATE           ,TKS_FXButton::ID_UPDATE         ,TKS_FXButton::onUpdate),
  FXMAPFUNC(SEL_ENTER            ,TKS_FXButton::ID_ENTER          ,TKS_FXButton::onEnter),
  FXMAPFUNC(SEL_LEAVE            ,TKS_FXButton::ID_LEAVE          ,TKS_FXButton::onLeave),
  FXMAPFUNC(SEL_FOCUSIN          ,TKS_FXButton::ID_FOCUSIN        ,TKS_FXButton::onFocusIn),
  FXMAPFUNC(SEL_FOCUSOUT         ,TKS_FXButton::ID_FOCUSOUT       ,TKS_FXButton::onFocusOut),
  FXMAPFUNC(SEL_UNGRABBED        ,TKS_FXButton::ID_UNGRABBED      ,TKS_FXButton::onUngrabbed),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS  ,TKS_FXButton::ID_LEFTBTNPRESS   ,TKS_FXButton::onLeftBtnPress),
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,TKS_FXButton::ID_LEFTBTNRELEASE ,TKS_FXButton::onLeftBtnRelease),
  FXMAPFUNC(SEL_KEYPRESS         ,TKS_FXButton::ID_KEYPRESS       ,TKS_FXButton::onKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE       ,TKS_FXButton::ID_KEYRELEASE     ,TKS_FXButton::onKeyRelease),
  FXMAPFUNC(SEL_KEYPRESS         ,FXWindow::ID_HOTKEY             ,TKS_FXButton::onHotKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE       ,FXWindow::ID_HOTKEY             ,TKS_FXButton::onHotKeyRelease),
  FXMAPFUNC(SEL_COMMAND          ,FXWindow::ID_CHECK              ,TKS_FXButton::onCheck),
  FXMAPFUNC(SEL_COMMAND          ,FXWindow::ID_UNCHECK            ,TKS_FXButton::onUncheck),
*/
/*
  FXMAPFUNC(SEL_UPDATE           ,TKS_FXButton::ID_SCRIPT ,TKS_FXButton::onUpdate),
  FXMAPFUNC(SEL_ENTER            ,TKS_FXButton::ID_SCRIPT ,TKS_FXButton::onEnter),
  FXMAPFUNC(SEL_LEAVE            ,TKS_FXButton::ID_SCRIPT ,TKS_FXButton::onLeave),
  FXMAPFUNC(SEL_FOCUSIN          ,TKS_FXButton::ID_SCRIPT ,TKS_FXButton::onFocusIn),
  FXMAPFUNC(SEL_FOCUSOUT         ,TKS_FXButton::ID_SCRIPT ,TKS_FXButton::onFocusOut),
  FXMAPFUNC(SEL_UNGRABBED        ,TKS_FXButton::ID_SCRIPT ,TKS_FXButton::onUngrabbed),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS  ,TKS_FXButton::ID_SCRIPT ,TKS_FXButton::onLeftBtnPress),
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,TKS_FXButton::ID_SCRIPT ,TKS_FXButton::onLeftBtnRelease),
  FXMAPFUNC(SEL_KEYPRESS         ,TKS_FXButton::ID_SCRIPT ,TKS_FXButton::onKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE       ,TKS_FXButton::ID_SCRIPT ,TKS_FXButton::onKeyRelease),
  FXMAPFUNC(SEL_KEYPRESS         ,FXWindow::ID_HOTKEY     ,TKS_FXButton::onHotKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE       ,FXWindow::ID_HOTKEY     ,TKS_FXButton::onHotKeyRelease),
  FXMAPFUNC(SEL_COMMAND          ,FXWindow::ID_CHECK      ,TKS_FXButton::onCheck),
  FXMAPFUNC(SEL_COMMAND          ,FXWindow::ID_UNCHECK    ,TKS_FXButton::onUncheck),
  */
  FXMAPFUNC(SEL_UPDATE           ,0 ,TKS_FXButton::onUpdate),
  FXMAPFUNC(SEL_ENTER            ,0 ,TKS_FXButton::onEnter),
  FXMAPFUNC(SEL_LEAVE            ,0 ,TKS_FXButton::onLeave),
  FXMAPFUNC(SEL_FOCUSIN          ,0 ,TKS_FXButton::onFocusIn),
  FXMAPFUNC(SEL_FOCUSOUT         ,0 ,TKS_FXButton::onFocusOut),
  FXMAPFUNC(SEL_UNGRABBED        ,0 ,TKS_FXButton::onUngrabbed),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS  ,0 ,TKS_FXButton::onLeftBtnPress),
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0 ,TKS_FXButton::onLeftBtnRelease),
  FXMAPFUNC(SEL_KEYPRESS         ,0 ,TKS_FXButton::onKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE       ,0 ,TKS_FXButton::onKeyRelease),
  FXMAPFUNC(SEL_KEYPRESS         ,FXWindow::ID_HOTKEY     ,TKS_FXButton::onHotKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE       ,FXWindow::ID_HOTKEY     ,TKS_FXButton::onHotKeyRelease),
  FXMAPFUNC(SEL_COMMAND          ,FXWindow::ID_CHECK      ,TKS_FXButton::onCheck),
  FXMAPFUNC(SEL_COMMAND          ,FXWindow::ID_UNCHECK    ,TKS_FXButton::onUncheck),

  };

FXIMPLEMENT(TKS_FXButton,FXButton,TKS_FXButtonMap,ARRAYNUMBER(TKS_FXButtonMap))

TKS_FXButton::TKS_FXButton(FXComposite *_p, sUI _opts) : FXButton(_p,"&Hello, World!",NULL,NULL/*application*/,0/*FXApp::ID_QUIT*/, _opts) {
	sc_onUpdate=0;
	sc_onEnter=0;
	sc_onLeave=0;
	sc_onFocusIn=0;
	sc_onFocusOut=0;
	sc_onUngrabbed=0;
	sc_onLeftBtnPress=0;
	sc_onLeftBtnRelease=0;
	sc_onKeyPress=0;
	sc_onKeyRelease=0;
	sc_onHotKeyPress=0;
	sc_onHotKeyRelease=0;
	sc_onCheck=0;
	sc_onUncheck=0;
}

long TKS_FXButton::onUpdate(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onUpdate\n");
	return FXButton::onUpdate(_o, _s, _e);
}

long TKS_FXButton::onEnter(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("onEnter()\n");
	return FXButton::onEnter(_o, _s, _e);
}

long TKS_FXButton::onLeave(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onLeave\n");
	return FXButton::onLeave(_o, _s, _e);
}

long TKS_FXButton::onFocusIn(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onFocusIn\n");
	return FXButton::onFocusIn(_o, _s, _e);
}

long TKS_FXButton::onFocusOut(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onFocusOut\n");
	return FXButton::onFocusOut(_o, _s, _e);
}

long TKS_FXButton::onUngrabbed(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onUngrabbed\n");
	return FXButton::onUngrabbed(_o, _s, _e);
}

long TKS_FXButton::onLeftBtnPress(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onLeftBtnPress\n");
	return FXButton::onLeftBtnPress(_o, _s, _e);
}

long TKS_FXButton::onLeftBtnRelease(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onLeftBtnRelease this=%08x\n", (sUI)this);
	long r=FXButton::onLeftBtnRelease(_o, _s, _e);
	if(sc_onLeftBtnRelease)
	{
		yac_host->yacEvalFunction(sc_onLeftBtnRelease, 0, 0);
		//return 1;
	}
	return r;
}

long TKS_FXButton::onKeyPress(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onKeyPress\n");
	return FXButton::onKeyPress(_o, _s, _e);
}

long TKS_FXButton::onKeyRelease(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onKeyRelease\n");
	return FXButton::onKeyRelease(_o, _s, _e);
}

long TKS_FXButton::onHotKeyPress(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onHotKeyPress\n");
	return FXButton::onHotKeyPress(_o, _s, _e);
}

long TKS_FXButton::onHotKeyRelease(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onHotKeyRelease\n");
	return FXButton::onHotKeyRelease(_o, _s, _e);
}

long TKS_FXButton::onCheck(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onCheck\n");
	return FXButton::onCheck(_o, _s, _e);
}

long TKS_FXButton::onUncheck(FXObject *_o, FXSelector _s, void *_e) {
	//yac_host->printf("FXButton::onUncheck\n");
	return FXButton::onUncheck(_o, _s, _e);
}




// ---------------------------------------------------------------------------------------------
class TKS_FXCheckButton : public FXCheckButton/*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXCheckButton)

public:	
	TKS_FXCheckButton(void) { }

	TKS_FXCheckButton(FXComposite *, sUI _opts);

	~TKS_FXCheckButton() {}

	long onLeftBtnRelease(FXObject*,FXSelector,void*);

public:
	void *sc_onLeftBtnRelease;

public:
	YAC_H(TKS_FXCheckButton);

	//void tksinter_setCheck(sSI);
	//void tksinter_getCheck(YAC_Value*_r);

#define YAC_CLASS TKS_FXCheckButton
#define YAC_NUM_COMMANDS 3

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setCheck"
#define YAC_CMD_1_RTTI 1
#define YAC_CMD_1_SYM tksinter_setCheck

#define YAC_CMD_2_NAME "getCheck"
#define YAC_CMD_2_RTTI 0
#define YAC_CMD_2_RET 1
#define YAC_CMD_2_SYM tksinter_getCheck

#define YAC_CMD_3_NAME "setText"
#define YAC_CMD_3_RTTI 3
#define YAC_CMD_3_SYM tksinter_setText

#define YAC_CMD_4_NAME "getText"
#define YAC_CMD_4_RTTI 0
#define YAC_CMD_4_RET 4
#define YAC_CMD_4_SYM tksinter_getText

#include <yac_createcommands.h>
};

YAC_C(TKS_FXCheckButton, "FXCheckButton");

void FXCheckButton::tksinter_setCheck(sSI _i) {
	setCheck(_i);
}

void FXCheckButton::tksinter_getCheck(YAC_Value *_r) {
	YAC_RETI(getCheck());
}

long TKS_FXCheckButton::onLeftBtnRelease(FXObject*_o,FXSelector _s,void*_e) {
	long r=FXCheckButton::onLeftBtnRelease(_o,_s,_e);
	if(sc_onLeftBtnRelease)
	{
		YAC_Value args[1];
		args[0].initInt(getCheck());
		yac_host->yacEvalFunction(sc_onLeftBtnRelease, 1, args);
	}
	return r;
}

FXDEFMAP(TKS_FXCheckButton) TKS_FXCheckButtonMap[]={
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0 ,TKS_FXCheckButton::onLeftBtnRelease),

  };

FXIMPLEMENT(TKS_FXCheckButton,FXCheckButton,TKS_FXCheckButtonMap,ARRAYNUMBER(TKS_FXCheckButtonMap))

TKS_FXCheckButton::TKS_FXCheckButton(FXComposite *_p, sUI _opts) : FXCheckButton(_p,"<<unset>>",NULL,NULL, _opts) {
	sc_onLeftBtnRelease=0;
}

// ---------------------------------------------------------------------------------------------
class TKS_FXComboBox : public FXComboBox {
public:
	FXDECLARE(TKS_FXComboBox)

public:	
	TKS_FXComboBox(void) { pane=0;button=0;list=0;field=0; }

	TKS_FXComboBox(FXComposite *, sUI _opts);

	~TKS_FXComboBox() {}

	long onListClicked(FXObject*,FXSelector,void*);

public:
	void *sc_onListClicked;
	sUI i_numItems;

public:

	YAC_H(TKS_FXComboBox);

	void tksinter_setCurrentItem(sSI);
	void tksinter_getCurrentItem(YAC_Value *_r);
	void tksinter_setEditable(sSI);
	void tksinter_setItems(YAC_Object*);
	void tksinter_getItemText(sSI, YAC_Value*);

#define YAC_CLASS TKS_FXComboBox
#define YAC_NUM_COMMANDS 6

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setCurrentItem"
#define YAC_CMD_1_RTTI 1
#define YAC_CMD_1_SYM tksinter_setCurrentItem

#define YAC_CMD_2_NAME "getCurrentItem"
#define YAC_CMD_2_RTTI 0
#define YAC_CMD_2_RET 1
#define YAC_CMD_2_SYM tksinter_getCurrentItem

#define YAC_CMD_3_NAME "setEditable"
#define YAC_CMD_3_RTTI 1
#define YAC_CMD_3_SYM tksinter_setEditable

#define YAC_CMD_4_NAME "setItems"
#define YAC_CMD_4_RTTI 3
#define YAC_CMD_4_SYM tksinter_setItems

#define YAC_CMD_5_NAME "getItemText"
#define YAC_CMD_5_RTTI 1
#define YAC_CMD_5_RET 4
#define YAC_CMD_5_SYM tksinter_getItemText

#include <yac_createcommands.h>
};

YAC_C(TKS_FXComboBox, "FXComboBox");

TKS_FXComboBox::TKS_FXComboBox(FXComposite *_p, sUI _opts) : FXComboBox(_p,0,0,0,0, _opts) {
	sc_onListClicked=0;
	i_numItems=0;
}

void TKS_FXComboBox::tksinter_setCurrentItem(sSI _i) {
	if(((sUI)_i)<i_numItems)
		setCurrentItem(_i);
	else
		yac_host->printf("[---] FXComboBox: item out of range (_i=%i numItems=%i)\n", _i, i_numItems);
}

void TKS_FXComboBox::tksinter_getCurrentItem(YAC_Value *_r) {
	YAC_RETI(getCurrentItem());
}

void TKS_FXComboBox::tksinter_setEditable(sSI _i) {
	setEditable(_i);
}

void TKS_FXComboBox::tksinter_getItemText(sSI _i, YAC_Value *_r) {
	YAC_String *s=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
	if( ((sUI)_i)<i_numItems)
	{
		FXString fxs=getItemText(_i);
		//yac_host->printf("item text=%s\n", fxs.text());
		YAC_String t; t.visit((char*)fxs.text());
		s->yacCopy(&t);
	}
	YAC_RETS(s,1);
}

void TKS_FXComboBox::tksinter_setItems(YAC_Object *_s) {
	if(YAC_CHK(_s, YAC_CLID_STRING))
	{
		YAC_String*s=(YAC_String*)_s;

		FXComboBox::clearItems();
		i_numItems=0;

		sUI i=0;
		//sUI j;
		sU8 *c=(sU8*)s->chars;
		sU8 *ce=c+s->length;
		YAC_String t;
		t.alloc(1024); t.length=1;
		sU8 *d=(sU8*)t.chars;
		sU8 *kc=c;
		while((c<ce)&&(i<1024))
		{
			if(*c == '\n')
			{
				// finish item
				d[i]=0;
				t.length=i+1;
				while(i--)
					d[i]=kc[i];
				FXComboBox::appendItem((char*)t.chars);
				i_numItems++;
				kc=c+1;
			}
			else
				if(*c == 0)
				{
					// finish item
					d[i]=0;
					t.length=i+1;
					while(i--)
						d[i]=kc[i];
					FXComboBox::appendItem((char*)t.chars);
					i_numItems++;
					kc=c+1;
				}
			i++;
			c++;
		}
		t.free();
	}
}

long TKS_FXComboBox::onListClicked(FXObject*_o,FXSelector _s,void*_e) {
	long r=FXComboBox::onListClicked(_o,_s,_e);
	if(sc_onListClicked)
	{
		YAC_Value args[1];
		args[0].initInt(getCurrentItem());
		yac_host->yacEvalFunction(sc_onListClicked, 1, args);
	}
	return r;
}

FXDEFMAP(TKS_FXComboBox) TKS_FXComboBoxMap[]={
	FXMAPFUNC(SEL_CLICKED,FXComboBox::ID_LIST, TKS_FXComboBox::onListClicked),
  };

FXIMPLEMENT(TKS_FXComboBox,FXComboBox,TKS_FXComboBoxMap,ARRAYNUMBER(TKS_FXComboBoxMap))



// ---------------------------------------------------------------------------------------------
class TKS_FXTextField : public FXTextField/*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXTextField)

public:	

	TKS_FXTextField(void) { s_temp=0; }

	TKS_FXTextField(FXComposite *, sUI _flags, sUI _ncols, char *_contents);

	~TKS_FXTextField() { if(s_temp) {yac_host->yacDelete(s_temp);s_temp=0;}}

	long onVerify(FXObject*,FXSelector,void*);
	long onKeyPress(FXObject*,FXSelector,void*);
	long onKeyRelease(FXObject*,FXSelector,void*);

public:
	void *sc_onVerify;
	void *sc_onKeyPress;
	void *sc_onKeyRelease;

	YAC_String *s_temp;

public:
	YAC_H(TKS_FXTextField);

	void tksinter_setText(YAC_Object *);
	void tksinter_getText(YAC_Value *);

#define YAC_CLASS TKS_FXTextField
#define YAC_NUM_COMMANDS 3

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setText"
#define YAC_CMD_1_RTTI 3
#define YAC_CMD_1_SYM tksinter_setText

#define YAC_CMD_2_NAME "getText"
#define YAC_CMD_2_RTTI 0
#define YAC_CMD_2_RET  4
#define YAC_CMD_2_SYM tksinter_getText

#include <yac_createcommands.h>

};

YAC_C(TKS_FXTextField, "FXTextField");

FXDEFMAP(TKS_FXTextField) TKS_FXTextFieldMap[]={
  FXMAPFUNC(SEL_VERIFY,0,TKS_FXTextField::onVerify),
  FXMAPFUNC(SEL_KEYPRESS,0,TKS_FXTextField::onKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE,0,TKS_FXTextField::onKeyRelease),
};

FXIMPLEMENT(TKS_FXTextField,FXTextField,TKS_FXTextFieldMap,ARRAYNUMBER(TKS_FXTextFieldMap))

TKS_FXTextField::TKS_FXTextField(FXComposite *_p, sUI _flags, sUI _ncols, char*_contents) : FXTextField(_p, _ncols, 0, 0, _flags, DEFAULT_PAD, DEFAULT_PAD, DEFAULT_PAD, DEFAULT_PAD) {
	sc_onVerify=0;
	sc_onKeyPress=0;
	sc_onKeyRelease=0;
	setText((const char*)_contents);
	s_temp=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
}

void TKS_FXTextField::tksinter_setText(YAC_Object *_s) {
	if(YAC_CHK(_s, YAC_CLID_STRING))
	{
		YAC_String *s=(YAC_String*)_s;
		//yac_host->printf("FXTextField::setText(%s)\n", s->chars);
		FXTextField::setText(s->chars);
	}
}

void TKS_FXTextField::tksinter_getText(YAC_Value *_r) {
	YAC_String *s=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
	FXString fxs=getText();
	
	s->copy((char*)fxs.text());
	YAC_RETS(s,1);
}

long TKS_FXTextField::onVerify(FXObject *_o, FXSelector _s,void*_e) {
	long r=FXTextField::onVerify(_o,_s,_e);
	if(sc_onVerify)
	{
		s_temp->visit((char*)(FXchar*)_e);
		YAC_Value args[1];
		args[0].initString(s_temp,0);
		yac_host->yacEvalFunction(sc_onVerify, 1, args);
		//return r;
	}
	return r;
}

long TKS_FXTextField::onKeyPress(FXObject *_o, FXSelector _s,void*_e) {
	long r=FXTextField::onKeyPress(_o, _s, _e);
	if(sc_onKeyPress)
	{
		FXEvent* event=(FXEvent*)_e;
		YAC_Value args[2];
		args[0].initInt(event->code);
		args[1].initInt(event->state);
		yac_host->yacEvalFunction(sc_onKeyPress, 2, args);
		//return 1;
	}
	return r;
}

long TKS_FXTextField::onKeyRelease(FXObject *_o, FXSelector _s,void*_e) {
	long r=FXTextField::onKeyRelease(_o, _s, _e);
	if(sc_onKeyRelease)
	{
		FXEvent* event=(FXEvent*)_e;
		YAC_Value args[2];
		args[0].initInt(event->code);
		args[1].initInt(event->state);
		yac_host->yacEvalFunction(sc_onKeyRelease, 2, args);
		//return 1;
	}
	return r;
}

// ---------------------------------------------------------------------------------------------
class TKS_FXSlider : public FXSlider/*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXSlider)

public:
	sSI onMotion_oldValue;

public:	
	TKS_FXSlider(void) { }

	TKS_FXSlider(FXComposite *, sUI _flags);

	~TKS_FXSlider() {}

	long onLeftBtnRelease(FXObject*,FXSelector,void*);
	long onMotion(FXObject*,FXSelector,void*);

public:
	void *sc_onLeftBtnRelease;
	void *sc_onMotion;
public:
	YAC_H(TKS_FXSlider);

	void tksinter_getValue(YAC_Value*);
	void tksinter_setHi(sSI);
	void tksinter_getHi(YAC_Value*);
	void tksinter_setLo(sSI);
	void tksinter_getLo(YAC_Value*);

#define YAC_CLASS TKS_FXSlider
#define YAC_NUM_COMMANDS 7

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setValue"
#define YAC_CMD_1_RTTI 1
#define YAC_CMD_1_SYM setValue

#define YAC_CMD_2_NAME "getValue"
#define YAC_CMD_2_RTTI 0
#define YAC_CMD_2_RET 1
#define YAC_CMD_2_SYM tksinter_getValue

#define YAC_CMD_3_NAME "setHi"
#define YAC_CMD_3_RTTI 1
#define YAC_CMD_3_SYM tksinter_setHi

#define YAC_CMD_4_NAME "getHi"
#define YAC_CMD_4_RTTI 0
#define YAC_CMD_4_RET 1
#define YAC_CMD_4_SYM tksinter_getHi

#define YAC_CMD_5_NAME "setLo"
#define YAC_CMD_5_RTTI 1
#define YAC_CMD_5_SYM tksinter_setLo

#define YAC_CMD_6_NAME "getLo"
#define YAC_CMD_6_RTTI 0
#define YAC_CMD_6_RET 1
#define YAC_CMD_6_SYM tksinter_getLo

#include <yac_createcommands.h>

};

YAC_C(TKS_FXSlider, "FXSlider");

FXDEFMAP(TKS_FXSlider) TKS_FXSliderMap[]={
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,TKS_FXSlider::onLeftBtnRelease),
  FXMAPFUNC(SEL_MOTION,0,TKS_FXSlider::onMotion),
};

FXIMPLEMENT(TKS_FXSlider,FXSlider,TKS_FXSliderMap,ARRAYNUMBER(TKS_FXSliderMap))

TKS_FXSlider::TKS_FXSlider(FXComposite *_p, sUI _flags) : FXSlider(_p, 0, 0, _flags) {
	sc_onLeftBtnRelease=0;
	sc_onMotion=0;
	onMotion_oldValue=-999999;
}

void TKS_FXSlider::tksinter_getValue(YAC_Value *_r) {
	YAC_RETI(getValue());
}

void TKS_FXSlider::tksinter_setLo(sSI _lo) {
	FXint lo,hi; getRange(lo,hi);
	setRange(_lo, hi);
}

void TKS_FXSlider::tksinter_setHi(sSI _hi) {
	FXint lo,hi; getRange(lo,hi);
	setRange(lo, _hi);
}

void TKS_FXSlider::tksinter_getLo(YAC_Value *_r) {
	FXint lo,hi; getRange(lo,hi);
	YAC_RETI(lo);
}

void TKS_FXSlider::tksinter_getHi(YAC_Value *_r) {
	FXint lo,hi; getRange(lo,hi);
	YAC_RETI(hi);
}

long TKS_FXSlider::onLeftBtnRelease(FXObject*_o,FXSelector _s,void* _e){
	//yac_host->printf("onLeftBtnRelease value=%i\n", getValue());
	long r=FXSlider::onLeftBtnRelease(_o,_s,_e);
	if(sc_onLeftBtnRelease)
	{
		YAC_String s;
		s.visit((char*)(FXchar*)_e);
		YAC_Value args[1];
		args[0].initInt(getValue());
		yac_host->yacEvalFunction(sc_onLeftBtnRelease, 1, args);
		//return 1;
	}
	return r;
}

long TKS_FXSlider::onMotion(FXObject*_o,FXSelector _s,void* _e){
	long r=FXSlider::onMotion(_o,_s,_e);
	sSI v=getValue();
	if(sc_onMotion&&(v!=onMotion_oldValue))
	{
		onMotion_oldValue=v;
		//yac_host->printf("onMotion value=%i\n", v);
		YAC_String s;
		s.visit((char*)(FXchar*)_e);
		YAC_Value args[1];
		args[0].initInt(getValue());
		yac_host->yacEvalFunction(sc_onMotion, 1, args);
		//return 1;
	}
	return r;
}

// ---------------------------------------------------------------------------------------------
class TKS_FXDial : public FXDial/*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXDial)

public:
	sSI onMotion_oldValue;

public:	
	TKS_FXDial(void) { }

	TKS_FXDial(FXComposite *, sUI _flags);

	~TKS_FXDial() {}

	long onLeftBtnRelease(FXObject*,FXSelector,void*);
	long onMotion(FXObject*,FXSelector,void*);

public:
	void *sc_onLeftBtnRelease;
	void *sc_onMotion;
public:
	YAC_H(TKS_FXDial);

	void tksinter_getValue(YAC_Value*);
	void tksinter_setHi(sSI);
	void tksinter_getHi(YAC_Value*);
	void tksinter_setLo(sSI);
	void tksinter_getLo(YAC_Value*);

#define YAC_CLASS TKS_FXDial
#define YAC_NUM_COMMANDS 7

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setValue"
#define YAC_CMD_1_RTTI 1
#define YAC_CMD_1_SYM setValue

#define YAC_CMD_2_NAME "getValue"
#define YAC_CMD_2_RTTI 0
#define YAC_CMD_2_RET 1
#define YAC_CMD_2_SYM tksinter_getValue

#define YAC_CMD_3_NAME "setHi"
#define YAC_CMD_3_RTTI 1
#define YAC_CMD_3_SYM tksinter_setHi

#define YAC_CMD_4_NAME "getHi"
#define YAC_CMD_4_RTTI 0
#define YAC_CMD_4_RET 1
#define YAC_CMD_4_SYM tksinter_getHi

#define YAC_CMD_5_NAME "setLo"
#define YAC_CMD_5_RTTI 1
#define YAC_CMD_5_SYM tksinter_setLo

#define YAC_CMD_6_NAME "getLo"
#define YAC_CMD_6_RTTI 0
#define YAC_CMD_6_RET 1
#define YAC_CMD_6_SYM tksinter_getLo

#include <yac_createcommands.h>

};

YAC_C(TKS_FXDial, "FXDial");

FXDEFMAP(TKS_FXDial) TKS_FXDialMap[]={
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,TKS_FXDial::onLeftBtnRelease),
  FXMAPFUNC(SEL_MOTION,0,TKS_FXDial::onMotion),
};

FXIMPLEMENT(TKS_FXDial,FXDial,TKS_FXDialMap,ARRAYNUMBER(TKS_FXDialMap))

TKS_FXDial::TKS_FXDial(FXComposite *_p, sUI _flags) : FXDial(_p, 0, 0, _flags) {
	sc_onLeftBtnRelease=0;
	sc_onMotion=0;
	onMotion_oldValue=-999999;
}

void TKS_FXDial::tksinter_getValue(YAC_Value *_r) {
	YAC_RETI(getValue());
}

void TKS_FXDial::tksinter_setLo(sSI _lo) {
	FXint lo,hi; getRange(lo,hi);
	setRange(_lo, hi);
}

void TKS_FXDial::tksinter_setHi(sSI _hi) {
	FXint lo,hi; getRange(lo,hi);
	setRange(lo, _hi);
}

void TKS_FXDial::tksinter_getLo(YAC_Value *_r) {
	FXint lo,hi; getRange(lo,hi);
	YAC_RETI(lo);
}

void TKS_FXDial::tksinter_getHi(YAC_Value *_r) {
	FXint lo,hi; getRange(lo,hi);
	YAC_RETI(hi);
}

long TKS_FXDial::onLeftBtnRelease(FXObject*_o,FXSelector _s,void* _e){
	//yac_host->printf("TKS_FXDial::onLeftBtnRelease value=%i\n", getValue());
	long r=FXDial::onLeftBtnRelease(_o,_s,_e);
	if(sc_onLeftBtnRelease)
	{
		YAC_String s;
		s.visit((char*)(FXchar*)_e);
		YAC_Value args[1];
		args[0].initInt(getValue());
		yac_host->yacEvalFunction(sc_onLeftBtnRelease, 1, args);
		//return 1;
	}
	return r;
}

long TKS_FXDial::onMotion(FXObject*_o,FXSelector _s,void* _e){
	long r=FXDial::onMotion(_o,_s,_e);
	sSI v=getValue();
	if(sc_onMotion&&(v!=onMotion_oldValue))
	{
		onMotion_oldValue=v;
		//yac_host->printf("TKS_FXDial::onMotion value=%i\n", v);
		YAC_String s;
		s.visit((char*)(FXchar*)_e);
		YAC_Value args[1];
		args[0].initInt(getValue());
		yac_host->yacEvalFunction(sc_onMotion, 1, args);
		//return 1;
	}
	return r;
}

// ---------------------------------------------------------------------------------------------
class TKS_FXScrollBar : public FXScrollBar/*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXScrollBar)

public:
	sSI onMotion_oldValue;

public:	
	TKS_FXScrollBar(void) { }

	TKS_FXScrollBar(FXComposite *, sUI _flags);

	~TKS_FXScrollBar() {}

	long onLeftBtnRelease(FXObject*,FXSelector,void*);
	long onMotion(FXObject*,FXSelector,void*);

public:
	void *sc_onLeftBtnRelease;
	void *sc_onMotion;
public:
	YAC_H(TKS_FXScrollBar);

	void tksinter_getValue(YAC_Value*);
	void tksinter_setHi(sSI);
	void tksinter_getHi(YAC_Value*);
	
#define YAC_CLASS TKS_FXScrollBar
#define YAC_NUM_COMMANDS 5

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setValue"
#define YAC_CMD_1_RTTI 1
#define YAC_CMD_1_SYM setPosition

#define YAC_CMD_2_NAME "getValue"
#define YAC_CMD_2_RTTI 0
#define YAC_CMD_2_RET 1
#define YAC_CMD_2_SYM tksinter_getValue

#define YAC_CMD_3_NAME "setHi"
#define YAC_CMD_3_RTTI 1
#define YAC_CMD_3_SYM tksinter_setHi

#define YAC_CMD_4_NAME "getHi"
#define YAC_CMD_4_RTTI 0
#define YAC_CMD_4_RET 1
#define YAC_CMD_4_SYM tksinter_getHi

#include <yac_createcommands.h>

};

YAC_C(TKS_FXScrollBar, "FXScrollBar");

FXDEFMAP(TKS_FXScrollBar) TKS_FXScrollBarMap[]={
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,TKS_FXScrollBar::onLeftBtnRelease),
  FXMAPFUNC(SEL_MOTION,0,TKS_FXScrollBar::onMotion),
};

FXIMPLEMENT(TKS_FXScrollBar,FXScrollBar,TKS_FXScrollBarMap,ARRAYNUMBER(TKS_FXScrollBarMap))

TKS_FXScrollBar::TKS_FXScrollBar(FXComposite *_p, sUI _flags) : FXScrollBar(_p, 0, 0, _flags) {
	sc_onLeftBtnRelease=0;
	sc_onMotion=0;
	onMotion_oldValue=-999999;
}

void TKS_FXScrollBar::tksinter_getValue(YAC_Value *_r) {
	YAC_RETI(getPosition());
}

void TKS_FXScrollBar::tksinter_setHi(sSI _hi) {
	setRange(_hi);
}

void TKS_FXScrollBar::tksinter_getHi(YAC_Value *_r) {
	YAC_RETI(getRange());
}

long TKS_FXScrollBar::onLeftBtnRelease(FXObject*_o,FXSelector _s,void* _e){
	//yac_host->printf("TKS_FXScrollBar::onLeftBtnRelease value=%i\n", getPosition());
	long r=FXScrollBar::onLeftBtnRelease(_o,_s,_e);
	if(sc_onLeftBtnRelease)
	{
		YAC_String s;
		s.visit((char*)(FXchar*)_e);
		YAC_Value args[1];
		args[0].initInt(getPosition());
		yac_host->yacEvalFunction(sc_onLeftBtnRelease, 1, args);
		//return 1;
	}
	return r;
}

long TKS_FXScrollBar::onMotion(FXObject*_o,FXSelector _s,void* _e){
	long r=FXScrollBar::onMotion(_o,_s,_e);
	sSI v=getPosition();
	if(sc_onMotion&&(v!=onMotion_oldValue))
	{
		onMotion_oldValue=v;
		//yac_host->printf("TKS_FXScrollBar::onMotion value=%i\n", v);
		YAC_String s;
		s.visit((char*)(FXchar*)_e);
		YAC_Value args[1];
		args[0].initInt(getPosition());
		yac_host->yacEvalFunction(sc_onMotion, 1, args);
		//return 1;
	}
	return r;
}

// ---------------------------------------------------------------------------------------------
/*class TKS_FXGroupBox : public FXGroupBox {
public:
	FXDECLARE(TKS_FXGroupBox)

public:	
	TKS_FXGroupBox(void) { }

	TKS_FXGroupBox(FXComposite *, sUI _flags);

	~TKS_FXGroupBox() {}


public:
	long onUncheckOther(FXObject*,FXSelector,void*);
};

FXDEFMAP(TKS_FXGroupBox) TKS_FXGroupBoxMap[]={
  FXMAPFUNC(SEL_UNCHECK_OTHER,0,TKS_FXGroupBox::onUncheckOther),
};

FXIMPLEMENT(TKS_FXGroupBox,FXGroupBox,TKS_FXGroupBoxMap,ARRAYNUMBER(TKS_FXGroupBoxMap))

TKS_FXGroupBox::TKS_FXGroupBox(FXComposite *_p, sUI _flags) : FXGroupBox(_p, "unset", _flags) {
//FXGroupBox(_p, sfx_unset, _flags, 0, 0, 0, 0, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING, DEFAULT_SPACING) {
}

long TKS_FXGroupBox::onUncheckOther(FXObject*_o,FXSelector _s, void*_e) {
	return FXGroupBox::onUncheckOther(_o, _s, _e);
}
*/ 
// ---------------------------------------------------------------------------------------------
class TKS_FXMenuCommand : public FXMenuCommand/*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXMenuCommand)

public:	
	TKS_FXMenuCommand(void) { }

	TKS_FXMenuCommand(FXComposite *, sUI _flags, char *_label);

	~TKS_FXMenuCommand() {}

	long onLeftBtnRelease(FXObject*,FXSelector,void*);
	long onRightBtnRelease(FXObject*,FXSelector,void*);

public:
	void *sc_onLeftBtnRelease;
	void *sc_onRightBtnRelease;
};

FXDEFMAP(TKS_FXMenuCommand) TKS_FXMenuCommandMap[]={
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,TKS_FXMenuCommand::onLeftBtnRelease),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,0,TKS_FXMenuCommand::onRightBtnRelease),
};

FXIMPLEMENT(TKS_FXMenuCommand,FXMenuCommand,TKS_FXMenuCommandMap,ARRAYNUMBER(TKS_FXMenuCommandMap))

TKS_FXMenuCommand::TKS_FXMenuCommand(FXComposite *_p, sUI _flags, char *_label) : FXMenuCommand(_p, _label, 0/*ic*/, 0, 0, _flags) {
	sc_onLeftBtnRelease=0;
	sc_onRightBtnRelease=0;
}

long TKS_FXMenuCommand::onLeftBtnRelease(FXObject*_o,FXSelector _s,void* _e){
	//yac_host->printf("TKS_FXMenuCommand::onLeftBtnRelease\n");
	if(sc_onLeftBtnRelease)
	{
		yac_host->yacEvalFunction(sc_onLeftBtnRelease, 0, 0);
		//return 1;
	}
	//long r=FXMenuCommand::onLeftBtnRelease(_o,_s,_e);
	long r=FXMenuCommand::onButtonRelease(_o,_s,_e);
	return r;
}

long TKS_FXMenuCommand::onRightBtnRelease(FXObject*_o,FXSelector _s,void* _e){
	//yac_host->printf("TKS_FXMenuCommand::onRightBtnRelease\n");
	//long r=FXMenuCommand::onRightBtnRelease(_o,_s,_e);
	long r=FXMenuCommand::onButtonRelease(_o,_s,_e);
	if(sc_onRightBtnRelease)
	{
		yac_host->yacEvalFunction(sc_onRightBtnRelease, 0, 0);
		//return 1;
	}
	return r;
}


// ---------------------------------------------------------------------------------------------
class TKS_FXGLCanvas : public FXGLCanvas /*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXGLCanvas)

public:
	FXTimer *timer;

public:	
	TKS_FXGLCanvas(void) { timer=0; }

	TKS_FXGLCanvas(FXComposite *, FXGLVisual *_vis, sUI _flags);

	~TKS_FXGLCanvas();

	long onKeyPress(FXObject*,FXSelector,void*);
	long onKeyRelease(FXObject*,FXSelector,void*);
	long onLeftBtnPress(FXObject*,FXSelector,void*);
	long onLeftBtnRelease(FXObject*,FXSelector,void*);
	long onMotion(FXObject*,FXSelector,void*);
	long onPaint(FXObject*,FXSelector,void*);
    long onTimeout(FXObject*,FXSelector,void*);

	sUI draw(void);


public:
	void *sc_onKeyPress;
	void *sc_onKeyRelease;
	void *sc_onLeftBtnPress;
	void *sc_onLeftBtnRelease;
	void *sc_onMotion;
	void *sc_onPaint;
	void *sc_onTimeout;

	sUI timer_lastInterval;

  enum{
    ID_TIMEOUT=FXMainWindow::ID_LAST
    };


public:
	YAC_H(TKS_FXGLCanvas);

	void tksinter_startTimer(sSI _ival);
	void tksinter_stopTimer(void);
	void tksinter_setTimeout(sSI _ival);

#define YAC_CLASS TKS_FXGLCanvas
#define YAC_NUM_COMMANDS 4

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "startTimer"
#define YAC_CMD_1_RTTI 1
#define YAC_CMD_1_SYM tksinter_startTimer

#define YAC_CMD_2_NAME "stopTimer"
#define YAC_CMD_2_RTTI 0
#define YAC_CMD_2_SYM tksinter_stopTimer

#define YAC_CMD_3_NAME "setTimeout"
#define YAC_CMD_3_RTTI 1
#define YAC_CMD_3_SYM tksinter_startTimer

#include <yac_createcommands.h>

};


void TKS_FXGLCanvas::tksinter_startTimer(sSI _ival) {
  tksinter_stopTimer();
  timer_lastInterval=_ival;
  //timer=getApp()->addTimeout(_ival,this,ID_TIMEOUT);
  timer=getApp()->addTimeout(this, ID_TIMEOUT, _ival, 0);
}

void TKS_FXGLCanvas::tksinter_stopTimer(void) {
	if(timer)
	{
		application->removeTimeout(timer);
		timer=0;
	}
}

YAC_C(TKS_FXGLCanvas, "FXGLCanvas");

FXDEFMAP(TKS_FXGLCanvas) TKS_FXGLCanvasMap[]={
  FXMAPFUNC(SEL_PAINT,0,TKS_FXGLCanvas::onPaint),
  FXMAPFUNC(SEL_MOTION,0,TKS_FXGLCanvas::onMotion),
  FXMAPFUNC(SEL_KEYPRESS,0,TKS_FXGLCanvas::onKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE,0,TKS_FXGLCanvas::onKeyRelease),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,TKS_FXGLCanvas::onLeftBtnPress),
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,TKS_FXGLCanvas::onLeftBtnRelease),
  FXMAPFUNC(SEL_TIMEOUT, TKS_FXGLCanvas::ID_TIMEOUT,  TKS_FXGLCanvas::onTimeout),
};

FXIMPLEMENT(TKS_FXGLCanvas,FXGLCanvas,TKS_FXGLCanvasMap,ARRAYNUMBER(TKS_FXGLCanvasMap))

TKS_FXGLCanvas::TKS_FXGLCanvas(FXComposite *_p, FXGLVisual *_vis, sUI _flags) : FXGLCanvas(_p, _vis, NULL, 0, _flags, 0, 0, 0, 0) {
	sc_onLeftBtnRelease=0;
	sc_onMotion=0;
	sc_onLeftBtnPress=0;
	sc_onPaint=0;
	sc_onKeyPress=0;
	sc_onKeyRelease=0;
	sc_onTimeout=0;
	timer_lastInterval=0;
	timer=0;
}

TKS_FXGLCanvas::~TKS_FXGLCanvas() {
	tksinter_stopTimer();
}

long TKS_FXGLCanvas::onPaint(FXObject*_o,FXSelector _s,void* _e) {
	long r=FXGLCanvas::onPaint(_o, _s, _e);
	draw();
	return r;
}

sUI TKS_FXGLCanvas::draw(void) {
	//yac_host->printf("onPaint w=%i h=%i\n", getWidth(), getHeight());

	GLdouble w = getWidth();
	GLdouble h = getHeight();
	GLdouble a = h>0 ? w/h : 1.0;
	  // Make context current
	makeCurrent();
	::glViewport(0,0,(sSI)w,(sSI)h);

	::glMatrixMode(GL_TEXTURE);
	::glLoadIdentity();
	::glMatrixMode(GL_MODELVIEW);
	::glLoadIdentity();
	::glClearColor(0,1,0,0);
	::glClear(GL_COLOR_BUFFER_BIT);

	if(sc_onPaint)
	{
		yac_host->yacEvalFunction(sc_onPaint, 0,0);
		//return 1;
	}
	
	// Swap if it is double-buffered
	if(fxglvisual->isDoubleBuffer())
		swapBuffers();
	
	// Make context non-current
	makeNonCurrent();
	return 1;
}

long TKS_FXGLCanvas::onLeftBtnRelease(FXObject*_o,FXSelector _s,void* _e) {
	long r=FXGLCanvas::onLeftBtnRelease(_o,_s,_e);
	if(sc_onLeftBtnRelease)
	{
		yac_host->yacEvalFunction(sc_onLeftBtnRelease, 0, 0);
		//return 1;
	}
	return r;
}

long TKS_FXGLCanvas::onLeftBtnPress(FXObject*_o,FXSelector _s,void* _e) {
	long r=FXGLCanvas::onLeftBtnPress(_o,_s,_e);
	if(sc_onLeftBtnPress)
	{
		yac_host->yacEvalFunction(sc_onLeftBtnPress, 0, 0);
		//return 1;
	}
	return r;
}

long TKS_FXGLCanvas::onMotion(FXObject*_o,FXSelector _s,void* _e){
	FXEvent *ev=(FXEvent*)_e;
	long r=FXGLCanvas::onMotion(_o,_s,_e);
	if(sc_onMotion)
	{
		YAC_Value args[2];
		args[0].initInt(ev->win_x);
		args[1].initInt(ev->win_y);
		//yac_host->printf("x=%i y=%i\n", args[0].value.int_val, args[1].value.int_val);
		yac_host->yacEvalFunction(sc_onMotion, 2, args);
		//return 1;
	}
	return r;
}

long TKS_FXGLCanvas::onKeyPress(FXObject *_o, FXSelector _s,void*_e) {
	//long r=
	if(sc_onKeyPress)
	{
		FXEvent* event=(FXEvent*)_e;
		YAC_Value args[2];
		args[0].initInt(event->code);
		args[1].initInt(event->state);
		yac_host->yacEvalFunction(sc_onKeyPress, 2, args);
		return 1;
	}
	else 
		return FXGLCanvas::onKeyPress(_o, _s, _e);
}

long TKS_FXGLCanvas::onKeyRelease(FXObject *_o, FXSelector _s,void*_e) {
	if(sc_onKeyRelease)
	{
		FXEvent* event=(FXEvent*)_e;
		YAC_Value args[2];
		args[0].initInt(event->code);
		args[1].initInt(event->state);
		yac_host->yacEvalFunction(sc_onKeyRelease, 2, args);
		return 1;
	}
	else
		return FXGLCanvas::onKeyRelease(_o, _s, _e);
}

long TKS_FXGLCanvas::onTimeout(FXObject *_o, FXSelector _s, void *_e) {
	tksinter_startTimer(timer_lastInterval);
	return draw();
}


// ---------------------------------------------------------------------------------------------
class TKS_FXMessageBox : public FXMessageBox/*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXMessageBox)

public:
	YAC_String *def_caption;
	YAC_String *def_message;

public:	
	TKS_FXMessageBox(void);

	TKS_FXMessageBox(FXComposite *, sUI _flags);

	~TKS_FXMessageBox();

	void initDefaults(void);

public:
	YAC_H(TKS_FXMessageBox);

	void tksinter_setCaption(YAC_Object*);
	void tksinter_setMessage(YAC_Object*);
	void tksinter_execute(YAC_Value *_r);


#define YAC_CLASS TKS_FXMessageBox
#define YAC_NUM_COMMANDS 4

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setCaption"
#define YAC_CMD_1_RTTI 3
#define YAC_CMD_1_SYM tksinter_setCaption

#define YAC_CMD_2_NAME "setMessage"
#define YAC_CMD_2_RTTI 3
#define YAC_CMD_2_SYM tksinter_setMessage

#define YAC_CMD_3_NAME "execute"
#define YAC_CMD_3_RTTI 0
#define YAC_CMD_3_RET 1
#define YAC_CMD_3_SYM tksinter_execute

#include <yac_createcommands.h>

};

TKS_FXMessageBox::TKS_FXMessageBox(void) {
	def_caption=0;
	def_message=0;
}

TKS_FXMessageBox::TKS_FXMessageBox(FXComposite *_p, sUI _flags) : FXMessageBox(_p, 0, 0, 0, _flags) {
	initDefaults();
}

TKS_FXMessageBox::~TKS_FXMessageBox() {
	if(def_caption)
	{
		delete def_caption;
		def_caption=0;
	}
	if(def_message)
	{
		delete def_message;
		def_message=0;
	}
}

void TKS_FXMessageBox::tksinter_setCaption(YAC_Object *_s) {
	if(YAC_CHK(_s, YAC_CLID_STRING))
	{
		YAC_String *s=(YAC_String*)_s;
		def_caption->yacCopy(s);
	}
}

void TKS_FXMessageBox::tksinter_setMessage(YAC_Object *_s) {
	if(YAC_CHK(_s, YAC_CLID_STRING))
	{
		YAC_String *s=(YAC_String*)_s;
		def_message->yacCopy(s);
	}
}

void TKS_FXMessageBox::tksinter_execute(YAC_Value *_r) {
	YAC_RETI(execute());
}

YAC_APIC void APIC_FXMessageBox_error(YAC_Object *_p, sSI _flags, YAC_Object *_c, YAC_Object *_m, YAC_Value *_r) {
	if(TKS_FXIsComposite(_p))
	if(YAC_CHK(_c, YAC_CLID_STRING))
	if(YAC_CHK(_m, YAC_CLID_STRING))
	{
		FXWindow *w=(FXWindow*)_p;
		if(_p)
		{
			YAC_RETI(
				FXMessageBox::error((FXWindow*)_p,  
				_flags?_flags:(DECOR_TITLE|DECOR_BORDER|MBOX_OK),
				(const char*)((YAC_String*)_c)->chars,
				(const char*)((YAC_String*)_m)->chars)
				);
		}
		else
		{
			YAC_RETI(FXMessageBox::error(application->getRootWindow(),  
				_flags?_flags:(DECOR_TITLE|DECOR_BORDER|MBOX_OK),
				(const char*)((YAC_String*)_c)->chars,
				(const char*)((YAC_String*)_m)->chars)
				);
		}
		return;

	}
	YAC_RETI(MBOX_CLICKED_QUIT);
}

YAC_APIC void APIC_FXMessageBox_question(YAC_Object *_p, sSI _flags, YAC_Object *_c, YAC_Object *_m, YAC_Value *_r) {
	if(TKS_FXIsComposite(_p))
	if(YAC_CHK(_c, YAC_CLID_STRING))
	if(YAC_CHK(_m, YAC_CLID_STRING))
	{
		if(_p)
		{
			YAC_RETI(
				FXMessageBox::question((FXWindow*)_p,
				_flags?_flags:(DECOR_TITLE|DECOR_BORDER|MBOX_OK),
				(const char*)((YAC_String*)_c)->chars,
				(const char*)((YAC_String*)_m)->chars)
				);
		}
		else
		{
			YAC_RETI(FXMessageBox::question(application->getRootWindow(),  
				_flags?_flags:(DECOR_TITLE|DECOR_BORDER|MBOX_OK),
				(const char*)((YAC_String*)_c)->chars,
				(const char*)((YAC_String*)_m)->chars)
				);
		}
		return;

	}
	YAC_RETI(MBOX_CLICKED_QUIT);
}

YAC_APIC void APIC_FXMessageBox_warning(YAC_Object *_p, sSI _flags, YAC_Object *_c, YAC_Object *_m, YAC_Value *_r) {
	if(TKS_FXIsComposite(_p))
	if(YAC_CHK(_c, YAC_CLID_STRING))
	if(YAC_CHK(_m, YAC_CLID_STRING))
	{
		if(_p)
		{
			YAC_RETI(
				FXMessageBox::warning((FXWindow*)_p,
				_flags?_flags:(DECOR_TITLE|DECOR_BORDER|MBOX_OK),
				(const char*)((YAC_String*)_c)->chars,
				(const char*)((YAC_String*)_m)->chars)
				);
		}
		else
		{
			YAC_RETI(FXMessageBox::warning(application->getRootWindow(),  
				_flags?_flags:(DECOR_TITLE|DECOR_BORDER|MBOX_OK),
				(const char*)((YAC_String*)_c)->chars,
				(const char*)((YAC_String*)_m)->chars)
				);
		}
		return;

	}
	YAC_RETI(MBOX_CLICKED_QUIT);
}

YAC_APIC void APIC_FXMessageBox_information(YAC_Object *_p, sSI _flags, YAC_Object *_c, YAC_Object *_m, YAC_Value *_r) {
	if(TKS_FXIsComposite(_p))
	if(YAC_CHK(_c, YAC_CLID_STRING))
	if(YAC_CHK(_m, YAC_CLID_STRING))
	{
		if(_p)
		{
			YAC_RETI(
				FXMessageBox::information((FXWindow*)_p,
				_flags?_flags:(DECOR_TITLE|DECOR_BORDER|MBOX_OK),
				(const char*)((YAC_String*)_c)->chars,
				(const char*)((YAC_String*)_m)->chars)
				);
		}
		else
		{
			YAC_RETI(FXMessageBox::information(application->getRootWindow(),  
				_flags?_flags:(DECOR_TITLE|DECOR_BORDER|MBOX_OK),
				(const char*)((YAC_String*)_c)->chars,
				(const char*)((YAC_String*)_m)->chars)
				);
		}
		return;

	}
	YAC_RETI(MBOX_CLICKED_QUIT);
}

void TKS_FXMessageBox::initDefaults(void) {
	def_caption=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
	def_message=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
}


YAC_C(TKS_FXMessageBox, "FXMessageBox");

FXIMPLEMENT(TKS_FXMessageBox,FXMessageBox,0,0)


// ---------------------------------------------------------------------------------------------
class TKS_FXFileDialog : public FXFileDialog /*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXFileDialog)

public:
	YAC_String *def_caption;
	YAC_String *def_message;

public:	
	TKS_FXFileDialog(void);

	TKS_FXFileDialog(FXComposite *, sUI _flags, char*_title);

	~TKS_FXFileDialog();

public:
	YAC_H(TKS_FXFileDialog);

	void tksinter_setFilename(YAC_Object*);
	void tksinter_setPattern(YAC_Object*);
	void tksinter_setPatternList(YAC_Object*);
	void tksinter_setCurrentPattern(sSI);
	void tksinter_setDirectory(YAC_Object*);
	void tksinter_setItemSpace(sSI);
	void tksinter_setFileBoxStyle(sSI);
	void tksinter_setSelectMode(sSI);
	void tksinter_setShowReadOnly(sSI);
	void tksinter_execute(YAC_Value*);

	void tksinter_getFilename(YAC_Value *_r);
	void tksinter_getFilenames(YAC_Value *_r);
	void tksinter_getDirectory(YAC_Value *_r);

#define YAC_CLASS TKS_FXFileDialog
#define YAC_NUM_COMMANDS 14

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setFilename"
#define YAC_CMD_1_RTTI 3
#define YAC_CMD_1_SYM tksinter_setFilename

#define YAC_CMD_2_NAME "setPattern"
#define YAC_CMD_2_RTTI 3
#define YAC_CMD_2_SYM tksinter_setPattern

#define YAC_CMD_3_NAME "setPatternList"
#define YAC_CMD_3_RTTI 3
#define YAC_CMD_3_SYM tksinter_setPatternList

#define YAC_CMD_4_NAME "setCurrentPattern"
#define YAC_CMD_4_RTTI 1
#define YAC_CMD_4_SYM tksinter_setCurrentPattern

#define YAC_CMD_5_NAME "setDirectory"
#define YAC_CMD_5_RTTI 3
#define YAC_CMD_5_SYM tksinter_setDirectory

#define YAC_CMD_6_NAME "setItemSpace"
#define YAC_CMD_6_RTTI 1
#define YAC_CMD_6_SYM tksinter_setItemSpace

#define YAC_CMD_7_NAME "setFileBoxStyle"
#define YAC_CMD_7_RTTI 1
#define YAC_CMD_7_SYM tksinter_setFileBoxStyle

#define YAC_CMD_8_NAME "setSelectMode"
#define YAC_CMD_8_RTTI 1
#define YAC_CMD_8_SYM tksinter_setSelectMode

#define YAC_CMD_9_NAME "setShowReadOnly"
#define YAC_CMD_9_RTTI 1
#define YAC_CMD_9_SYM tksinter_setShowReadOnly

#define YAC_CMD_10_NAME "execute"
#define YAC_CMD_10_RTTI 0
#define YAC_CMD_10_RET 1
#define YAC_CMD_10_SYM tksinter_execute

#define YAC_CMD_11_NAME "getFilename"
#define YAC_CMD_11_RTTI 0
#define YAC_CMD_11_RET 4
#define YAC_CMD_11_SYM tksinter_getFilename

#define YAC_CMD_12_NAME "getDirectory"
#define YAC_CMD_12_RTTI 0
#define YAC_CMD_12_RET 4
#define YAC_CMD_12_SYM tksinter_getDirectory

#define YAC_CMD_13_NAME "getFilenames"
#define YAC_CMD_13_RTTI 0
#define YAC_CMD_13_RET 4
#define YAC_CMD_13_SYM tksinter_getFilenames

#include <yac_createcommands.h>

};

TKS_FXFileDialog::TKS_FXFileDialog(void) {
}

TKS_FXFileDialog::~TKS_FXFileDialog() {
}

TKS_FXFileDialog::TKS_FXFileDialog(FXComposite *_p, sUI _flags, char *_title) : FXFileDialog(_p, _title, _flags) {
}

FXIMPLEMENT(TKS_FXFileDialog, FXFileDialog,0,0)

YAC_C(TKS_FXFileDialog, "FXFileDialog");


void TKS_FXFileDialog::tksinter_setFilename(YAC_Object*_s) {
	if(YAC_CHK(_s, YAC_CLID_STRING))
	{
		YAC_String*s=(YAC_String*)_s;
		FXFileDialog::setFilename((const char*)s->chars);
	}
}

void TKS_FXFileDialog::tksinter_setPattern(YAC_Object*_s) {
	if(YAC_CHK(_s, YAC_CLID_STRING))
	{
		YAC_String*s=(YAC_String*)_s;
		FXFileDialog::setPattern((const char*)s->chars);
	}
}

void TKS_FXFileDialog::tksinter_setPatternList(YAC_Object*_s) {
	if(YAC_CHK(_s, YAC_CLID_STRING))
	{
		YAC_String*s=(YAC_String*)_s;
		FXFileDialog::setPatternList((const char*)s->chars);
	}
}

void TKS_FXFileDialog::tksinter_setCurrentPattern(sSI _i) {
	FXFileDialog::setCurrentPattern(_i);
}

void TKS_FXFileDialog::tksinter_setDirectory(YAC_Object*_s) {
	if(YAC_CHK(_s, YAC_CLID_STRING))
	{
		YAC_String*s=(YAC_String*)_s;
		FXFileDialog::setDirectory((const char*)s->chars);
	}
}

void TKS_FXFileDialog::tksinter_setItemSpace(sSI _i) {
	FXFileDialog::setItemSpace(_i);
}

void TKS_FXFileDialog::tksinter_setFileBoxStyle(sSI _i) {
	FXFileDialog::setFileBoxStyle(_i);
}

void TKS_FXFileDialog::tksinter_setSelectMode(sSI _i) {
	FXFileDialog::setSelectMode(_i);
}

void TKS_FXFileDialog::tksinter_setShowReadOnly(sSI _i) {
	FXFileDialog::showReadOnly((_i!=0));
}

void TKS_FXFileDialog::tksinter_execute(YAC_Value *_r) {
	YAC_RETI(FXFileDialog::execute());
}

void TKS_FXFileDialog::tksinter_getFilename(YAC_Value *_r) {
	YAC_String *s=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
	FXString fxs=FXFileDialog::getFilename();
	s->copy((char*)fxs.text());
	YAC_RETS(s,1);
}

void TKS_FXFileDialog::tksinter_getDirectory(YAC_Value *_r) {
	YAC_String *s=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
	FXString fxs=FXFileDialog::getDirectory();
	s->copy((char*)fxs.text());
	YAC_RETS(s,1);
}

void TKS_FXFileDialog::tksinter_getFilenames(YAC_Value *_r) {
	YAC_Object *sa=yac_host->yacNewByID(YAC_CLID_STRINGARRAY);
	FXString *sl=FXFileDialog::getFilenames();
	if(sl)
	{
		YAC_String ts; ts.class_ID=YAC_CLID_STRING;
		FXString *tsl=sl;
		int i=0;
		while(tsl->text()[0])
		{
			i++;
			tsl++;
		}
		sa->yacArrayAlloc(i,1,0,0);

		YAC_Value t;
		t.initObject(&ts, 0);
		while(i--)
		{
			tsl--;
			ts.visit((char*)tsl->text());
			sa->yacArraySet(i, &t);
		}
	}
	YAC_RETO(sa,1);
}


// ---------------------------------------------------------------------------------------------
class TKS_FX4Splitter : public FX4Splitter {
public:
	FXDECLARE(TKS_FX4Splitter)

public:	
	TKS_FX4Splitter(void) { }

	TKS_FX4Splitter(FXComposite *, sUI _flags);

	~TKS_FX4Splitter() {}

public:
	YAC_H(TKS_FX4Splitter);

	void tksinter_setHSplit(sSI);
	void tksinter_setVSplit(sSI);
	void tksinter_setSplitterStyle(sSI);
	void tksinter_setBarSize(sSI);
	void tksinter_setExpanded(sSI);
	void tksinter_getExpanded(YAC_Value *_r);

#define YAC_CLASS TKS_FX4Splitter
#define YAC_NUM_COMMANDS 7

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setHSplit"
#define YAC_CMD_1_RTTI 1
#define YAC_CMD_1_SYM tksinter_setHSplit

#define YAC_CMD_2_NAME "setVSplit"
#define YAC_CMD_2_RTTI 1
#define YAC_CMD_2_SYM tksinter_setVSplit

#define YAC_CMD_3_NAME "setSplitterStyle"
#define YAC_CMD_3_RTTI 1
#define YAC_CMD_3_SYM tksinter_setSplitterStyle

#define YAC_CMD_4_NAME "setBarSize"
#define YAC_CMD_4_RTTI 1
#define YAC_CMD_4_SYM tksinter_setBarSize

#define YAC_CMD_5_NAME "setExpanded"
#define YAC_CMD_5_RTTI 1
#define YAC_CMD_5_SYM tksinter_setExpanded

#define YAC_CMD_6_NAME "getExpanded"
#define YAC_CMD_6_RTTI 0
#define YAC_CMD_6_RET 1
#define YAC_CMD_6_SYM tksinter_getExpanded

#include <yac_createcommands.h>

};

FXIMPLEMENT(TKS_FX4Splitter,FX4Splitter,0,0);
YAC_C(TKS_FX4Splitter, "FX4Splitter");


TKS_FX4Splitter::TKS_FX4Splitter(FXComposite *_p, sUI _flags) : FX4Splitter(_p, _flags) {
}

void TKS_FX4Splitter::tksinter_setHSplit(sSI _i) {
	FX4Splitter::setHSplit(_i);
}

void TKS_FX4Splitter::tksinter_setVSplit(sSI _i) {
	FX4Splitter::setVSplit(_i);
}

void TKS_FX4Splitter::tksinter_setSplitterStyle(sSI _i) {
	FX4Splitter::setSplitterStyle(_i);
}

void TKS_FX4Splitter::tksinter_setBarSize(sSI _i) {
	FX4Splitter::setBarSize(_i);
}

void TKS_FX4Splitter::tksinter_setExpanded(sSI _i) {
	FX4Splitter::setExpanded(_i);
}

void TKS_FX4Splitter::tksinter_getExpanded(YAC_Value *_r) {
	YAC_RETI(FX4Splitter::getExpanded());
}

 
// ---------------------------------------------------------------------------------------------
class TKS_FXColorDialog : public FXColorDialog /*, public YAC_Object*/ {
public:
	FXDECLARE(TKS_FXColorDialog)

public:	
	TKS_FXColorDialog(void);

	TKS_FXColorDialog(FXComposite *, sUI _flags, char*_title);

	~TKS_FXColorDialog();

public:
	YAC_H(TKS_FXColorDialog);

	void tksinter_setRgba(sSI);
	void tksinter_getRgba(YAC_Value*);
	void tksinter_setOpaqueOnly(sSI);
	void tksinter_execute(YAC_Value*);
	void tksinter_getRgbaString(YAC_Value*);

#define YAC_CLASS TKS_FXColorDialog
#define YAC_NUM_COMMANDS 6

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "setRgba"
#define YAC_CMD_1_RTTI 1
#define YAC_CMD_1_SYM tksinter_setRgba

#define YAC_CMD_2_NAME "getRgba"
#define YAC_CMD_2_RTTI 0
#define YAC_CMD_2_RET 1
#define YAC_CMD_2_SYM tksinter_getRgba

#define YAC_CMD_3_NAME "setOpaqueOnly"
#define YAC_CMD_3_RTTI 1
#define YAC_CMD_3_SYM tksinter_setOpaqueOnly

#define YAC_CMD_4_NAME "execute"
#define YAC_CMD_4_RTTI 0
#define YAC_CMD_4_RET 1
#define YAC_CMD_4_SYM tksinter_execute

#define YAC_CMD_5_NAME "getRgbaString"
#define YAC_CMD_5_RTTI 0
#define YAC_CMD_5_RET 4
#define YAC_CMD_5_SYM tksinter_getRgbaString

#include <yac_createcommands.h>

};

TKS_FXColorDialog::TKS_FXColorDialog(void) {
}

TKS_FXColorDialog::~TKS_FXColorDialog() {
}

TKS_FXColorDialog::TKS_FXColorDialog(FXComposite *_p, sUI _flags, char *_title) : FXColorDialog(_p, _title, _flags) {
}

FXIMPLEMENT(TKS_FXColorDialog, FXColorDialog,0,0)

YAC_C(TKS_FXColorDialog, "FXColorDialog");

void TKS_FXColorDialog::tksinter_setRgba(sSI _i) {
	setRGBA(FXRGBA( ((_i>>16)&0xFF), ((_i>>8)&0xFF), (_i&0xFF), ((_i>>24)&0xFF) ));
}

void TKS_FXColorDialog::tksinter_getRgba(YAC_Value*_r) {
	FXColor fxc=getRGBA();
	//yac_host->printf("fxc=%08x\n", (sUI)fxc);
	sUI i_rgba;

	i_rgba=
		( (fxc&0x000000FF)<<16 )|
		( (fxc&0x0000FF00)     )|
		( (fxc&0x00FF0000)>>16 )|
		( (fxc&0xFF000000)     );
	//yac_host->printf("rgba=%08x\n", (sUI)i_rgba);

		// foxes uses a different byte order (abgr) for packed rgba colors  
	    // (((FXuint)(FXuchar)(r)<<24) | ((FXuint)(FXuchar)(g)<<16) | ((FXuint)(FXuchar)(b)<<8) | ((FXuint)(FXuchar)(a)))

	YAC_RETI(i_rgba);
}

void TKS_FXColorDialog::tksinter_getRgbaString(YAC_Value *_r) {
	YAC_String*s=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
	s->alloc(10);
	YAC_Value r;tksinter_getRgba(&r);
	::sprintf(s->chars,"#%08x", r.value.int_val);
	YAC_RETS(s,1);
}

void TKS_FXColorDialog::tksinter_setOpaqueOnly(sSI _i) {
	setOpaqueOnly(_i);
}
void TKS_FXColorDialog::tksinter_execute(YAC_Value*_r) {
	YAC_RETI(FXColorDialog::execute());
}


// ---------------------------------------------------------------------------------------------


class FXPageRelocEntry {
public:
	YAC_String  fn_name;
	void      **sc_function;
	YAC_Object *fx_object;

	FXPageRelocEntry(void) { sc_function=0; fx_object=0; }
	~FXPageRelocEntry() { }
};

class _TKS_FXPage : public YAC_Object {
	public:
		YAC_Object *elements; // labeled elements
		YAC_Object *font_elements; // 
		YAC_Object *composite_elements;
		void       *script; // YAC_Script with temp functions (callbacks)
		YAC_String  t_script; // 
		YAC_String  s_globals; // FXButton fx_mybutton; ..

		sUI         script_tag_index;
		YAC_String *script_tags[TKS_FXMAXSCRIPTS];

		YAC_String  kw_unset;
		YAC_String  kw_script;
		YAC_String  kw_fxmainwindow;
		YAC_String  kw_fxlabel;
		YAC_String  kw_fxbutton;
		YAC_String  kw_fxfont;
		YAC_String  kw_fxtextfield;
		YAC_String  kw_fxslider;
		YAC_String  kw_fxmenubar;
		YAC_String  kw_fxmenupane;
		YAC_String  kw_fxmenutitle;
		YAC_String  kw_fxmenucommand;
		YAC_String  kw_fxhorizontalframe;
		YAC_String  kw_fxverticalframe;
		YAC_String  kw_fxdial;
		YAC_String  kw_fxscrollbar;
		YAC_String  kw_fxgroupbox;
		YAC_String  kw_fxhorizontalseparator;
		YAC_String  kw_fxverticalseparator;
		YAC_String  kw_fxglcanvas;
		YAC_String  kw_fxmessagebox;
		YAC_String  kw_fxfiledialog;
		YAC_String  kw_fx4splitter;
		YAC_String  kw_fxcolordialog;
		YAC_String  kw_fxcheckbutton;
		YAC_String  kw_fxcombobox;

		YAC_String  cbs_onUpdate;
		YAC_String  cbs_onEnter;
		YAC_String  cbs_onLeave;
		YAC_String  cbs_onFocusIn;
		YAC_String  cbs_onFocusOut;
		YAC_String  cbs_onUngrabbed;
		YAC_String  cbs_onLeftBtnPress;
		YAC_String  cbs_onLeftBtnRelease;
		YAC_String  cbs_onKeyPress;
		YAC_String  cbs_onKeyRelease;
		YAC_String  cbs_onHotKeyPress;
		YAC_String  cbs_onHotKeyRelease;
		YAC_String  cbs_onCheck;
		YAC_String  cbs_onUncheck;
		YAC_String  cbs_onVerify;
		YAC_String  cbs_onMotion;
		YAC_String  cbs_onRightBtnRelease;
		YAC_String  cbs_onTimeout;
		YAC_String  cbs_onPaint;
		YAC_String  cbs_onListClicked;
		
		YAC_String  ats_title;
		YAC_String  ats_x;
		YAC_String  ats_y;
		YAC_String  ats_w;
		YAC_String  ats_h;
		YAC_String  ats_hSpacing;
		YAC_String  ats_vSpacing;
		YAC_String  ats_padLeft;
		YAC_String  ats_padRight;
		YAC_String  ats_padTop;
		YAC_String  ats_padBottom;
		YAC_String  ats_icon;
		YAC_String  ats_packingHints;
		YAC_String  ats_text;
		YAC_String  ats_helpText;
		YAC_String  ats_tipText;
		YAC_String  ats_justify;
		YAC_String  ats_iconPosition;
		YAC_String  ats_frameStyle;
		YAC_String  ats_layoutHints;
		YAC_String  ats_flags;
		YAC_String  ats_id;
		YAC_String  ats_size;
		YAC_String  ats_face;
		YAC_String  ats_weight;
		YAC_String  ats_slant;
		YAC_String  ats_encoding;
		YAC_String  ats_setWidth;
		YAC_String  ats_font;
		YAC_String  ats_cols;
		YAC_String  ats_orientation;
		YAC_String  ats_headSize;
		YAC_String  ats_slotSize;
		YAC_String  ats_lo;
		YAC_String  ats_hi;
		YAC_String  ats_value;
		YAC_String  ats_inc;
		YAC_String  ats_tickDelta;
		YAC_String  ats_label;
		YAC_String  ats_parent;
		YAC_String  ats_revolutionIncrement;
		YAC_String  ats_notchSpacing;
		YAC_String  ats_notchOffset;
		YAC_String  ats_dialStyle;
		YAC_String  ats_page;
		YAC_String  ats_groupBoxStyle;
		YAC_String  ats_timeout;
		YAC_String  ats_body;
		YAC_String  ats_caption;
		YAC_String  ats_message;
		YAC_String  ats_filename;
		YAC_String  ats_pattern;
		YAC_String  ats_patternList;
		YAC_String  ats_currentPattern;
		YAC_String  ats_directory;
		YAC_String  ats_itemSpace;
		YAC_String  ats_fileBoxStyle;
		YAC_String  ats_selectMode;
		YAC_String  ats_showReadOnly;
		YAC_String  ats_hSplit;
		YAC_String  ats_vSplit;
		YAC_String  ats_splitterStyle;
		YAC_String  ats_barSize;
		YAC_String  ats_expanded;
		YAC_String  ats_rgba;
		YAC_String  ats_opaqueOnly;
		YAC_String  ats_check;
		YAC_String  ats_checkButtonStyle;
		YAC_String  ats_comboStyle;
		YAC_String  ats_numVisible;
		YAC_String  ats_currentItem;
		YAC_String  ats_editable;
		YAC_String  ats_numColumns;
		YAC_String  ats_items;

		YAC_String  os_function;
		YAC_String  os_voidargs;
		//YAC_String  os_intargs;
		//YAC_String  os_intintargs;
		//YAC_String  os_floatargs;
		//YAC_String  os_stringargs;
		YAC_String  os_endfunction;

		cobjectptr  fxstack[TKS_FXSTACKSIZE];
		sSI         fxstackp;
		sSI         fnid;
		//cobjectptr  fxprev;
		FXComposite*getParentComposite(void) { if(c_forcedparent)return c_forcedparent;else/* if(fxprev.fxobject)return fxprev.fxcomposite; else */return fxstack[fxstackp].fxcomposite; }

		FXMenuBar  *c_fxmenubar;
		FXComposite*c_fxcomposite;
		FXComposite*c_forcedparent;
		//FXMenuPane *c_fxmenupane;
		YAC_String *c_sid;
		YAC_String  c_sfnid;

		YAC_String    *cb_odecl;
		YAC_String    *cb_oname;
		//CBMTableEntry *cb_mtbl;


		cobjectptr  cobject;
		//YAC_Object *cobject_yac;
		sUI         c_opts;
		sBool       b_defopt;
		
		FXPageRelocEntry reloc[TKS_FXRELOCSIZE];
		sUI              reloci;

	public:
		YAC_H(_TKS_FXPage);

		_TKS_FXPage(void);
		~_TKS_FXPage();

		void    parseScript(YAC_Object *_hash);
		void    parseFXLabel(YAC_Object *_hash);
		void    parseFXButton(YAC_Object *_hash);
		void    parseFXFont(YAC_Object *_hash);
		void    parseFXTextField(YAC_Object *_hash);
		void    parseFXSlider(YAC_Object *_hash);
		void    parseFXMenuBar(YAC_Object *_hash, YAC_TreeNode *_t);
		void    parseFXMenuPane(YAC_Object *_hash, YAC_TreeNode *_t);
		void    parseFXMenuTitle(YAC_Object *_hash);
		void    parseFXMenuCommand(YAC_Object *_hash);
		void    parseFXHorizontalFrame(YAC_Object *_hash, YAC_TreeNode *_t);
		void    parseFXVerticalFrame(YAC_Object *_hash, YAC_TreeNode *_t);
		void    parseFXDial(YAC_Object *_hash);
		void    parseFXScrollBar(YAC_Object *_hash);
		void    parseFXGroupBox(YAC_Object *_hash, YAC_TreeNode *_t);
		void    parseFXHorizontalSeparator(YAC_Object *_hash);
		void    parseFXVerticalSeparator(YAC_Object *_hash);
		void    parseFXGLCanvas(YAC_Object *_hash);
		void    parseFXMessageBox(YAC_Object *_hash);
		void    parseFXFileDialog(YAC_Object *_hash);
		void    parseFX4Splitter(YAC_Object *_hash, YAC_TreeNode *_t);
		void    parseFXColorDialog(YAC_Object *_hash);
		void    parseFXCheckButton(YAC_Object *_hash);
		void    parseFXComboBox(YAC_Object *_hash);

		void    parseFXFrameArgs(YAC_Object *_hash);
		void    parseFXWindowArgs(YAC_Object *_hash);
		void    parseFXLabelArgs(YAC_Object *_hash);


		void    parseNode(YAC_TreeNode *);
		//void    parseID(YAC_Object *_hash);
		void    parseCompositeID(YAC_Object *_hash);

		void    tksinter_createFromTree(YAC_Object*, YAC_Value *_r);
		void    pushFX(void);
		void    popFX(void);
		void    beginScript(void);
		void    endScript(YAC_String*, void**);
		void    addScript(void**_fn, YAC_String *_s, YAC_String *_args);
		void    parseCb(YAC_Object *_hash, void **_cbk, YAC_String *_name, YAC_String *_args);
		FXFont *findFontElement(YAC_String*_s);
		FXComposite*findCompositeElement(YAC_String*_s);

#define YAC_CLASS _TKS_FXPage
#define YAC_NUM_COMMANDS 2

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "createFromTree"
#define YAC_CMD_1_RTTI 3
#define YAC_CMD_1_RET 1
#define YAC_CMD_1_SYM tksinter_createFromTree

	#include <yac_createcommands.h>
};
YAC_C(_TKS_FXPage, "FXPage");

_TKS_FXPage::_TKS_FXPage(void) {
	int i;
	for(i=0; i<TKS_FXSTACKSIZE; i++)
		fxstack[i].fxobject=0;
	fxstackp=TKS_FXSTACKSIZE;
	fnid=0;
	reloci=0;
	cobject.fxobject=0;
	c_fxmenubar=0;
	c_fxcomposite=0;
	script=0;
	elements=0;
	font_elements=0;
	composite_elements=0;
	c_forcedparent=0;
	c_sid=0;
	script_tag_index=0;
}

_TKS_FXPage::~_TKS_FXPage() {
	if(script)
	{
		yac_host->yacDeleteModule(script);
		script=0;
	}
	if(elements)
	{
		yac_host->yacDelete(elements);
		elements=0;
	}
	if(font_elements)
	{
		yac_host->yacDelete(font_elements);
		font_elements=0;
	}
	if(composite_elements)
	{
		yac_host->yacDelete(composite_elements);
		composite_elements=0;
	}
}

void _TKS_FXPage::pushFX(void) {
	if(fxstackp>0)
		fxstack[--fxstackp].fxobject=cobject.fxobject;
}

void _TKS_FXPage::popFX(void) {
	if(fxstackp!=TKS_FXSTACKSIZE)
		cobject.fxobject=fxstack[fxstackp++].fxobject;
}

void _TKS_FXPage::beginScript(void) {
	static char csfnid[256]; ::sprintf(csfnid, "cb_%03i", fnid++);
	c_sfnid.visit(csfnid);
	t_script.append(&os_function);
	t_script.append(&c_sfnid);
}

void _TKS_FXPage::endScript(YAC_String *_s, void**_fn) {
	t_script.append(cb_odecl);
	t_script.append(&cb_odecl_fxname);
	/*if(cb_mtbl)
	{
		CBMTableEntry *e=cb_mtbl;
		while(e->name.chars)
		{
			yac_host->yacStringReplace(_s, &e->name, &e->rname);
			e++;
		}
	}*/
	t_script.append(_s);
	t_script.append(&os_endfunction);
	
	FXPageRelocEntry *r=&reloc[reloci++];
	r->fn_name.copy(c_sfnid.chars);
	r->fx_object=(YAC_Object*)cobject.fxobject;
	r->sc_function=_fn;
}

void _TKS_FXPage::addScript(void **_fn, YAC_String *_s, YAC_String *_args) {
	beginScript();
	t_script.append(_args);
	endScript(_s, _fn);
	//yac_host->printf("addScript(%s)\n",_s->chars);
}

sBool _TKS_FXPage_GetHashIntVal(YAC_Object *_hash, YAC_String *_name, sSI *_r) {
	YAC_Value r;
	_hash->yacHashGet(_name, &r); // return value of hash is never read/write
	switch(r.type)
	{
	case 1:
		*_r=r.value.int_val;
		return 1;
	case 2:
		*_r=(sSI)r.value.float_val;
		return 1;
	case 3:
		*_r=(r.value.object_val!=0);
		return 1;
	case 4:
		return yac_host->yacStringScan(r.value.string_val, _r);
	}
	return 0;
}

sBool _TKS_FXPage_GetHashStringVal(YAC_Object *_hash, YAC_String *_name, YAC_String **_r) {
	YAC_Value r;
	_hash->yacHashGet(_name, &r); // return value of hash is never read/write
	if(r.type==4)
		if(r.value.string_val)
		{
			*_r=r.value.string_val;
			return 1;
		}
	return 0;
}

void _TKS_FXPage::parseCb(YAC_Object *_hash, void **_cbk, YAC_String *_name, YAC_String *_args) {
	YAC_Value r;
	_hash->yacHashGet(_name, &r);
	if(r.type==4)
		if(r.value.string_val)
		{
			addScript(_cbk, r.value.string_val, _args);
		}
}

void _TKS_FXPage::parseFXWindowArgs(YAC_Object *_hash) {
	sSI ival;
	if(_TKS_FXPage_GetHashIntVal(_hash, &ats_x, &ival))
		cobject.fxwindow->setX(ival);
	if(_TKS_FXPage_GetHashIntVal(_hash, &ats_y, &ival))
		cobject.fxwindow->setY(ival);
	if(_TKS_FXPage_GetHashIntVal(_hash, &ats_w, &ival))
		cobject.fxwindow->setWidth(ival);
	if(_TKS_FXPage_GetHashIntVal(_hash, &ats_h, &ival))
		cobject.fxwindow->setHeight(ival);
	parseFXFrameArgs(_hash);
}

void _TKS_FXPage::parseFXLabelArgs(YAC_Object *_h) {
	YAC_String *s;
	//sSI i;
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_iconPosition, &s))
		cobject.fxlabel->setIconPosition(yac_host->yacScanFlags(s));
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_justify, &s))
		cobject.fxlabel->setJustify(yac_host->yacScanFlags(s));
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_text, &s))
		cobject.fxlabel->setText(s->chars);
	else
		cobject.fxlabel->setText("<<unset>>");

	if(_TKS_FXPage_GetHashStringVal(_h, &ats_font, &s))
	{
		FXFont *f=findFontElement(s);
		if(f)
			cobject.fxlabel->setFont(f);
	}

	parseFXWindowArgs(_h);
	// textcolor
}

void _TKS_FXPage::parseFXFrameArgs(YAC_Object *_h) {
	YAC_String *s;
	sSI i_pad;
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_frameStyle, &s))
		cobject.fxframe->setFrameStyle(yac_host->yacScanFlags(s));
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_layoutHints, &s))
		cobject.fxframe->setLayoutHints(yac_host->yacScanFlags(s));

	if(_TKS_FXPage_GetHashIntVal(_h, &ats_padLeft, &i_pad))
		cobject.fxframe->setPadLeft(i_pad);
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_padRight, &i_pad))
		cobject.fxframe->setPadRight(i_pad);
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_padTop, &i_pad))
		cobject.fxframe->setPadTop(i_pad);
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_padBottom, &i_pad))
		cobject.fxframe->setPadBottom(i_pad);

	if(_TKS_FXPage_GetHashStringVal(_h, &ats_helpText, &s))
		cobject.fxlabel->setHelpText(s->chars);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_tipText, &s))
		cobject.fxlabel->setTipText(s->chars);
}

void _TKS_FXPage::parseFXMenuBar(YAC_Object *_h, YAC_TreeNode *_t) {
	//yac_host->printf("parseFXMenuBar\n");
	cobject.fxmenubar=new FXMenuBar(c_fxcomposite, b_defopt?(LAYOUT_SIDE_TOP|LAYOUT_FILL_X):c_opts);
	//parseID(_h);
	c_fxmenubar=cobject.fxmenubar;
	if(_t->right)
	{
		pushFX();
		parseNode(_t->right);
		popFX();
	}
}

void _TKS_FXPage::parseFXMenuPane(YAC_Object *_h, YAC_TreeNode *_t) {
	//yac_host->printf("parseFXMenuPane\n");
	cobject.fxmenupane=new FXMenuPane(c_fxcomposite);
	//parseID(_h);
	if(_t->right)
	{
		pushFX();
		parseNode(_t->right);
		popFX();
		//yac_host->printf("parseFXMenuPane done\n");
	}
}

void _TKS_FXPage::parseFXMenuCommand(YAC_Object *_h) {
	//yac_host->printf("parseFXMenuCommand\n");
	YAC_String *s_label=&kw_unset;
	_TKS_FXPage_GetHashStringVal(_h, &ats_label, &s_label);

	cobject.tksfxmenucommand=new TKS_FXMenuCommand(fxstack[fxstackp].fxmenupane, c_opts, s_label->chars);
	//cobject_yac=(YAC_Object*)cobject.tksfxmenucommand;
	//cobject.fxmenucommand->class_ID=clid_FXMenuCommand;
	parseCb(_h, &cobject.tksfxmenucommand->sc_onLeftBtnRelease, &cbs_onLeftBtnRelease, &os_voidargs);
	parseCb(_h, &cobject.tksfxmenucommand->sc_onRightBtnRelease, &cbs_onRightBtnRelease, &os_voidargs);
	//parseID(_h);
}

void _TKS_FXPage::parseFXMenuTitle(YAC_Object *_h) {
	//yac_host->printf("parseFXMenuTitle\n");
	YAC_String *s_label; s_label=&kw_unset;
	_TKS_FXPage_GetHashStringVal(_h, &ats_label, &s_label);
	cobject.fxmenutitle=new FXMenuTitle(c_fxmenubar, s_label->chars, 0, fxstack[fxstackp].fxmenupane);
	//parseID(_h);
	//yac_host->printf("//parseFXMenuTitle\n");
}


void _TKS_FXPage::parseFXHorizontalFrame(YAC_Object *_h, YAC_TreeNode *_t) {
	//yac_host->printf("parseFXHorizontalFrame()\n");
	cobject.fxhorizontalframe=new FXHorizontalFrame(getParentComposite(),b_defopt?(LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y):c_opts,0,0,0,0, 0,0,0,0);
	parseFXWindowArgs(_h);
	parseCompositeID(_h);
	if(_t->right)
	{
		pushFX();
		parseNode(_t->right);
		popFX();
		//yac_host->printf("parseFXHorizontalFrame done\n");
	}
}

void _TKS_FXPage::parseFXVerticalFrame(YAC_Object *_h, YAC_TreeNode *_t) {
	//yac_host->printf("parseFXVerticalFrame()\n");
	cobject.fxverticalframe=new FXVerticalFrame(getParentComposite(),b_defopt?(FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT):c_opts,0,0,0,0, 0,0,0,0);
	parseFXWindowArgs(_h);
	parseCompositeID(_h);
	if(_t->right)
	{
		pushFX();
		parseNode(_t->right);
		popFX();
		//yac_host->printf("parseFXVerticalFrame done\n");
	}
}

void _TKS_FXPage::parseFXGroupBox(YAC_Object *_h, YAC_TreeNode *_t) {
	//yac_host->printf("parseFXGroupBox()\n");
	YAC_String *s_label=&kw_unset;
	_TKS_FXPage_GetHashStringVal(_h, &ats_text, &s_label);

	cobject.fxgroupbox=new FXGroupBox(getParentComposite(), s_label->chars, b_defopt?(GROUPBOX_NORMAL|FRAME_RIDGE):c_opts);

	parseFXWindowArgs(_h);
	parseCompositeID(_h);

	YAC_String *s;
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_font, &s))
	{
		FXFont *f=findFontElement(s);
		if(f) 
			cobject.fxgroupbox->setFont(f);
	}

	if(_TKS_FXPage_GetHashStringVal(_h, &ats_groupBoxStyle, &s))
		cobject.fxgroupbox->setGroupBoxStyle(yac_host->yacScanFlags(s));

	if(_t->right)
	{
		pushFX();
		parseNode(_t->right);
		popFX();
		//yac_host->printf("parseFXGroupBox done\n");
	}
}

void _TKS_FXPage::parseFXHorizontalSeparator(YAC_Object *_h) {
	cobject.fxhorizontalseparator=new FXHorizontalSeparator(getParentComposite(), b_defopt?(SEPARATOR_GROOVE|LAYOUT_FILL_X):c_opts);
	parseFXWindowArgs(_h);
	parseCompositeID(_h);
}

void _TKS_FXPage::parseFXVerticalSeparator(YAC_Object *_h) {
	cobject.fxverticalseparator=new FXVerticalSeparator(getParentComposite(), b_defopt?(SEPARATOR_GROOVE|LAYOUT_FILL_Y):c_opts);
	parseFXWindowArgs(_h);
	parseCompositeID(_h);
}

void _TKS_FXPage::parseFXGLCanvas(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxglcanvas;
	cb_oname=&kw_fxglcanvas;
	//cb_mtbl =cb_mtbl_fxglcanvas;

	sSI i_timeout=0;
	cobject.tksfxglcanvas=new TKS_FXGLCanvas(getParentComposite(), GetFXGLVisual(), b_defopt?0:c_opts);
	parseFXWindowArgs(_h);
	parseCb(_h, &cobject.tksfxglcanvas->sc_onPaint         , &cbs_onPaint         , &os_voidargs);
	parseCb(_h, &cobject.tksfxglcanvas->sc_onMotion        , &cbs_onMotion        , &cb_args_fxglcanvas_onMotion);
	parseCb(_h, &cobject.tksfxglcanvas->sc_onKeyPress      , &cbs_onKeyPress      , &cb_args_fxtextfield_onKey);
	parseCb(_h, &cobject.tksfxglcanvas->sc_onKeyRelease    , &cbs_onKeyRelease    , &cb_args_fxtextfield_onKey);
	parseCb(_h, &cobject.tksfxglcanvas->sc_onLeftBtnPress  , &cbs_onLeftBtnPress  , &os_voidargs);
	parseCb(_h, &cobject.tksfxglcanvas->sc_onLeftBtnRelease, &cbs_onLeftBtnRelease, &os_voidargs);
	parseCb(_h, &cobject.tksfxglcanvas->sc_onTimeout       , &cbs_onTimeout       , &os_voidargs);
	_TKS_FXPage_GetHashIntVal(_h, &ats_timeout, &i_timeout);
	if(i_timeout)
		cobject.tksfxglcanvas->tksinter_startTimer(i_timeout);
	parseCompositeID(_h);
}

void _TKS_FXPage::parseFXMessageBox(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxmessagebox;
	cb_oname=&kw_fxmessagebox;

	YAC_String *s;

	sUI fl=b_defopt?DECOR_TITLE|DECOR_BORDER:c_opts;
	FXComposite *p=getParentComposite();
	cobject.tksfxmessagebox=new TKS_FXMessageBox(p, fl);
	cobject.tksfxmessagebox->class_ID=clid_FXMessageBox;
		//(getParentComposite(), 0, 0, 0, b_defopt?DECOR_TITLE|DECOR_BORDER:c_opts);
	//yac_host->printf("[...] new FXMessageBox(p=%08x flags=%08x)\n", (sUI)p, fl);
	parseFXWindowArgs(_h);

	if(_TKS_FXPage_GetHashStringVal(_h, &ats_caption, &s))
		cobject.tksfxmessagebox->tksinter_setCaption(s);

	if(_TKS_FXPage_GetHashStringVal(_h, &ats_message, &s))
		cobject.tksfxmessagebox->tksinter_setMessage(s);

	parseCompositeID(_h);
}

void _TKS_FXPage::parseFXFileDialog(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxfiledialog;
	cb_oname=&kw_fxfiledialog;

	YAC_String *s;
	sSI i_currentPattern=0;
	sSI i_itemSpace=1;
	sSI i_showReadOnly=0;
	YAC_String *s_title=&kw_unset;

	_TKS_FXPage_GetHashStringVal(_h, &ats_title, &s_title);

	FXComposite *p=getParentComposite();
	cobject.tksfxfiledialog=new TKS_FXFileDialog(p, c_opts, s_title->chars);
	cobject.tksfxfiledialog->class_ID=clid_FXFileDialog;
	//yac_host->printf("[...] new FXFileDialog(p=%08x flags=%08x)\n", (sUI)p, c_opts);
	parseFXWindowArgs(_h);

	if(_TKS_FXPage_GetHashStringVal(_h, &ats_filename, &s))
		cobject.tksfxfiledialog->tksinter_setFilename(s);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_pattern, &s))
		cobject.tksfxfiledialog->tksinter_setPattern(s);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_patternList, &s))
		cobject.tksfxfiledialog->tksinter_setPatternList(s);
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_currentPattern, &i_currentPattern))
		cobject.tksfxfiledialog->tksinter_setCurrentPattern(i_currentPattern);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_directory, &s))
		cobject.tksfxfiledialog->tksinter_setDirectory(s);
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_itemSpace, &i_itemSpace))
		cobject.tksfxfiledialog->tksinter_setItemSpace(i_itemSpace);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_fileBoxStyle, &s))
		cobject.tksfxfiledialog->setFileBoxStyle(yac_host->yacScanFlags(s));
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_selectMode, &s))
		cobject.tksfxfiledialog->setSelectMode(yac_host->yacScanFlags(s));
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_showReadOnly, &i_showReadOnly))
		cobject.tksfxfiledialog->tksinter_setShowReadOnly(i_showReadOnly);

	parseCompositeID(_h);
}

void _TKS_FXPage::parseFX4Splitter(YAC_Object *_h, YAC_TreeNode *_t) {
	cb_odecl=&cb_odecl_fx4splitter;
	cb_oname=&kw_fx4splitter;

	cobject.tksfx4splitter=new TKS_FX4Splitter(getParentComposite(), b_defopt?(FOURSPLITTER_NORMAL):c_opts);
	cobject.tksfx4splitter->class_ID=clid_FX4Splitter;

	parseFXWindowArgs(_h);
	parseCompositeID(_h);

	YAC_String *s=0;
	sSI i_hSplit=0;
	sSI i_vSplit=0;
	sSI i_barSize=0;
	sSI i_expanded=0;

	if(_TKS_FXPage_GetHashIntVal(_h, &ats_hSplit, &i_hSplit))
		cobject.tksfx4splitter->setHSplit(i_hSplit);
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_vSplit, &i_vSplit))
		cobject.tksfx4splitter->setVSplit(i_vSplit);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_splitterStyle, &s))
		cobject.tksfx4splitter->setSplitterStyle(yac_host->yacScanFlags(s));
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_barSize, &i_barSize))
		cobject.tksfx4splitter->setBarSize(i_barSize);
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_expanded, &i_expanded))
		cobject.tksfx4splitter->setExpanded(i_expanded);

	if(_t->right)
	{
		pushFX();
		parseNode(_t->right);
		popFX();
		//yac_host->printf("parseFX4Splitterdone\n");
	}
}

void _TKS_FXPage::parseFXColorDialog(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxcolordialog;
	cb_oname=&kw_fxcolordialog;

	sSI i_opaqueOnly=0;
	sSI i_rgba=((0xffd78623)>>1)&0x7f7f7f7f;
	YAC_String *s_title=&kw_unset;

	_TKS_FXPage_GetHashStringVal(_h, &ats_title, &s_title);

	FXComposite *p=getParentComposite();
	cobject.tksfxcolordialog=new TKS_FXColorDialog(p, c_opts, s_title->chars);
	cobject.tksfxcolordialog->class_ID=clid_FXColorDialog;
	//yac_host->printf("[...] new FXColorDialog(p=%08x flags=%08x)\n", (sUI)p, c_opts);
	parseFXWindowArgs(_h);

	_TKS_FXPage_GetHashIntVal(_h, &ats_rgba, &i_rgba);

	cobject.tksfxcolordialog->tksinter_setRgba(i_rgba);
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_opaqueOnly, &i_opaqueOnly))
		cobject.tksfxcolordialog->setOpaqueOnly(i_opaqueOnly);

	parseCompositeID(_h);
}

void _TKS_FXPage::parseFXCheckButton(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxcheckbutton;
	cb_oname=&kw_fxcheckbutton;

	YAC_String *s;
	sSI i_check=0;
	sSI i_checkButtonStyle=0;

	FXComposite *p=getParentComposite();
	cobject.tksfxcheckbutton=new TKS_FXCheckButton(p, b_defopt?CHECKBUTTON_NORMAL:c_opts);
	cobject.tksfxcheckbutton->class_ID=clid_FXCheckButton;

	if(_TKS_FXPage_GetHashIntVal(_h, &ats_check, &i_check)) 
		cobject.tksfxcheckbutton->tksinter_setCheck(i_check);

	if(_TKS_FXPage_GetHashStringVal(_h, &ats_checkButtonStyle, &s))
		cobject.tksfxcheckbutton->setCheckButtonStyle(yac_host->yacScanFlags(s));

	parseCb(_h, &cobject.tksfxcheckbutton->sc_onLeftBtnRelease , &cbs_onLeftBtnRelease , &cb_args_fxcheckbutton_onLeftBtnRelease);

	//yac_host->printf("[...] new FXCheckButton(p=%08x flags=%08x)\n", (sUI)p, c_opts);
	parseFXLabelArgs(_h);

	parseCompositeID(_h);
}

void _TKS_FXPage::parseFXComboBox(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxcombobox;
	cb_oname=&kw_fxcombobox;

	YAC_String *s;
	sSI i=0;

	FXComposite *p=getParentComposite();
	cobject.tksfxcombobox=new TKS_FXComboBox(p, b_defopt?COMBOBOX_NORMAL:c_opts);
	cobject.tksfxcombobox->class_ID=clid_FXComboBox;

	if(_TKS_FXPage_GetHashStringVal(_h, &ats_font, &s))
	{
		FXFont *f=findFontElement(s);
		if(f)
			cobject.tksfxcombobox->setFont(f);
	}
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_comboStyle, &s))
		cobject.tksfxcombobox->setComboStyle(yac_host->yacScanFlags(s));
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_numVisible, &i)) 
		cobject.tksfxcombobox->setNumVisible(i);
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_numColumns, &i)) 
		cobject.tksfxcombobox->setNumColumns(i);
	if(_TKS_FXPage_GetHashIntVal(_h, &ats_editable, &i)) 
		cobject.tksfxcombobox->setEditable(i);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_text, &s))
		cobject.tksfxcombobox->setText(s->chars);
	//else
		//cobject.fxlabel->setText("");

	if(_TKS_FXPage_GetHashStringVal(_h, &ats_items, &s))
		cobject.tksfxcombobox->tksinter_setItems(s);

	parseCb(_h, &cobject.tksfxcombobox->sc_onListClicked, &cbs_onListClicked, &cb_args_fxcombobox_onListClicked);

	//yac_host->printf("[...] new FXCheckButton(p=%08x flags=%08x)\n", (sUI)p, c_opts);
	parseFXWindowArgs(_h);

	parseCompositeID(_h);
}

void _TKS_FXPage::parseScript(YAC_Object *_h) {
	YAC_String *s;
	if(script_tag_index!=TKS_FXMAXSCRIPTS)
	{
		if(_TKS_FXPage_GetHashStringVal(_h, &ats_body, &s))
			script_tags[script_tag_index++]=s;
	}
}

void _TKS_FXPage::parseFXLabel(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxlabel;
	cb_oname=&kw_fxlabel;
	//cb_mtbl =0;//cb_mtbl_fxlabel;

	cobject.tksfxlabel=new TKS_FXLabel(getParentComposite(), b_defopt?LABEL_NORMAL:c_opts);
	cobject.tksfxlabel->class_ID=clid_FXLabel;
	//cobject_yac=(YAC_Object*)cobject.tksfxlabel;
	parseFXLabelArgs(_h);
	parseCompositeID(_h);
}

void _TKS_FXPage::parseFXButton(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxbutton;
	cb_oname=&kw_fxbutton;
	//cb_mtbl =cb_mtbl_fxbutton;

	cobject.tksfxbutton=new TKS_FXButton(getParentComposite(), b_defopt?BUTTON_NORMAL:c_opts);
	//yac_host->printf("newButton adr=%08x\n", (sUI) cobject.fxbutton);
	cobject.tksfxbutton->class_ID=clid_FXButton;
	//cobject_yac=(YAC_Object*)cobject.tksfxbutton;
	parseFXLabelArgs(_h);

	parseCb(_h, &cobject.tksfxbutton->sc_onLeftBtnRelease, &cbs_onLeftBtnRelease, &os_voidargs);
	parseCompositeID(_h);
}

void _TKS_FXPage::parseFXTextField(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxtextfield;
	cb_oname=&kw_fxtextfield;
	//cb_mtbl =cb_mtbl_fxtextfield;

	//YAC_String *s;
	sSI i_cols=24;
	YAC_String s_tcontent;
	YAC_String *s_content=&s_tcontent;

	_TKS_FXPage_GetHashIntVal(_h, &ats_cols, &i_cols);
	_TKS_FXPage_GetHashStringVal(_h, &ats_text, &s_content);

	sUI i_fl=b_defopt?TEXTFIELD_NORMAL:c_opts;
	//yac_host->printf("[...] new FXTextField(cols=%i flags=%08x content=%s)\n", i_cols, i_fl, s_content->chars);

	cobject.tksfxtextfield=new TKS_FXTextField(getParentComposite(), i_fl, i_cols, s_content->chars);
	cobject.tksfxtextfield->class_ID=clid_FXTextField;
	//cobject_yac=(YAC_Object*)cobject.tksfxtextfield;
	parseCb(_h, &cobject.tksfxtextfield->sc_onVerify    , &cbs_onVerify    , &cb_args_fxtextfield_onVerify);
	parseCb(_h, &cobject.tksfxtextfield->sc_onKeyPress  , &cbs_onKeyPress  , &cb_args_fxtextfield_onKey);
	parseCb(_h, &cobject.tksfxtextfield->sc_onKeyRelease, &cbs_onKeyRelease, &cb_args_fxtextfield_onKey);
	parseFXWindowArgs(_h);
	parseCompositeID(_h);
}

void _TKS_FXPage::parseFXSlider(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxslider;
	cb_oname=&kw_fxslider;
	//cb_mtbl =cb_mtbl_fxslider;

	sSI i_headSize=6;
	sSI i_slotSize=8;
	sSI i_tickDelta=1;
	sSI i_value=0;
	sSI i_lo=0;
	sSI i_hi=100;
	sSI i_inc=1;

	_TKS_FXPage_GetHashIntVal(_h, &ats_headSize, &i_headSize);
	_TKS_FXPage_GetHashIntVal(_h, &ats_slotSize, &i_slotSize);
	_TKS_FXPage_GetHashIntVal(_h, &ats_tickDelta, &i_tickDelta);
	_TKS_FXPage_GetHashIntVal(_h, &ats_value, &i_value);
	_TKS_FXPage_GetHashIntVal(_h, &ats_inc, &i_inc);
	_TKS_FXPage_GetHashIntVal(_h, &ats_lo, &i_lo);
	_TKS_FXPage_GetHashIntVal(_h, &ats_hi, &i_hi);

	sUI i_fl=b_defopt?SLIDER_NORMAL:c_opts;
	//yac_host->printf("[...] new FXSlider(flags=%08x)\n", i_fl);

	cobject.tksfxslider=new TKS_FXSlider(getParentComposite(), i_fl);
	cobject.tksfxslider->class_ID=clid_FXSlider;
	//cobject_yac=(YAC_Object*)cobject.tksfxslider;

	cobject.tksfxslider->setHeadSize(i_headSize);
	cobject.tksfxslider->setSlotSize(i_slotSize);
	cobject.tksfxslider->setTickDelta(i_tickDelta);
	cobject.tksfxslider->setRange(i_lo, i_hi);
	cobject.tksfxslider->setValue(i_value);
	cobject.tksfxslider->setIncrement(i_inc);

	parseCb(_h, &cobject.tksfxslider->sc_onLeftBtnRelease, &cbs_onLeftBtnRelease, &cb_args_fxslider_onChange);
	parseCb(_h, &cobject.tksfxslider->sc_onMotion, &cbs_onMotion, &cb_args_fxslider_onChange);
	parseFXWindowArgs(_h);
	parseCompositeID(_h);
}

void _TKS_FXPage::parseFXDial(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxdial;
	cb_oname=&kw_fxdial;
	//cb_mtbl =cb_mtbl_fxdial;

	sSI i_cols=24;

	sSI i_revolutionIncrement=360;
	sSI i_notchSpacing=10;
	sSI i_notchOffset=0;
	sSI i_dialStyle=0;	YAC_String *s_dialstyle;
	sSI i_value=0;
	sSI i_lo=0;
	sSI i_hi=3600;

	_TKS_FXPage_GetHashIntVal(_h, &ats_revolutionIncrement, &i_revolutionIncrement);
	_TKS_FXPage_GetHashIntVal(_h, &ats_notchSpacing, &i_notchSpacing);
	_TKS_FXPage_GetHashIntVal(_h, &ats_notchOffset, &i_notchOffset);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_dialStyle, &s_dialstyle))
		i_dialStyle=yac_host->yacScanFlags(s_dialstyle);
	_TKS_FXPage_GetHashIntVal(_h, &ats_value, &i_value);
	_TKS_FXPage_GetHashIntVal(_h, &ats_lo, &i_lo);
	_TKS_FXPage_GetHashIntVal(_h, &ats_hi, &i_hi);

	sUI i_fl=b_defopt?DIAL_NORMAL:c_opts;
	//yac_host->printf("[...] new FXDial(revInc=%i notchSpacing=%i notchOffset=%i dialStyle=%i value=%i lo=%i hi=%i flags=%08x)\n", 
	//		i_revolutionIncrement, i_notchSpacing, i_notchOffset, i_dialStyle, i_value, i_lo, i_hi, i_fl);

	cobject.tksfxdial=new TKS_FXDial(getParentComposite(), i_fl);
	cobject.tksfxdial->class_ID=clid_FXDial;
	//cobject_yac=(YAC_Object*)cobject.tksfxdial;

	cobject.tksfxdial->setRevolutionIncrement(i_revolutionIncrement);
	cobject.tksfxdial->setNotchSpacing(i_notchSpacing);
	cobject.tksfxdial->setNotchOffset(i_notchOffset);
	cobject.tksfxdial->setDialStyle(i_dialStyle);
	cobject.tksfxdial->setRange(i_lo, i_hi);
	cobject.tksfxdial->setValue(i_value);

	parseCb(_h, &cobject.tksfxdial->sc_onLeftBtnRelease, &cbs_onLeftBtnRelease, &cb_args_fxslider_onChange);
	parseCb(_h, &cobject.tksfxdial->sc_onMotion        , &cbs_onMotion        , &cb_args_fxslider_onChange);
	parseFXWindowArgs(_h);
	parseCompositeID(_h);
}

void _TKS_FXPage::parseFXScrollBar(YAC_Object *_h) {
	cb_odecl=&cb_odecl_fxscrollbar;
	cb_oname=&kw_fxscrollbar;
	//cb_mtbl=cb_mtbl_fxscrollbar;

	sSI i_value=0;
	sSI i_hi=10;
	sSI i_inc=1;
	sSI i_page=1;

	_TKS_FXPage_GetHashIntVal(_h, &ats_page, &i_page);
	_TKS_FXPage_GetHashIntVal(_h, &ats_value, &i_value);
	_TKS_FXPage_GetHashIntVal(_h, &ats_inc, &i_inc);
	_TKS_FXPage_GetHashIntVal(_h, &ats_hi, &i_hi);

	sUI i_fl=b_defopt?SCROLLBAR_HORIZONTAL:c_opts;
	//yac_host->printf("[...] new FXScrollBar(flags=%08x value=%i hi=%i inc=%i page=%i)\n", i_fl, i_value, i_hi, i_inc, i_page);

	cobject.tksfxscrollbar=new TKS_FXScrollBar(getParentComposite(), i_fl);
	cobject.tksfxscrollbar->class_ID=clid_FXScrollBar;
	//cobject_yac=(YAC_Object*)cobject.tksfxscrollbar;

	cobject.tksfxscrollbar->setPage(i_page);
	cobject.tksfxscrollbar->setRange(i_hi);
	cobject.tksfxscrollbar->setPosition(i_value);
	cobject.tksfxscrollbar->setLine(i_inc);

	parseCb(_h, &cobject.tksfxscrollbar->sc_onLeftBtnRelease, &cbs_onLeftBtnRelease, &cb_args_fxslider_onChange);
	parseCb(_h, &cobject.tksfxscrollbar->sc_onMotion        , &cbs_onMotion        , &cb_args_fxslider_onChange);
	parseFXWindowArgs(_h);
	parseCompositeID(_h);
}

FXFont *_TKS_FXPage::findFontElement(YAC_String*_s) {
	YAC_Value r;
	yac_host->printf("findFontElement(%s)\n", (char*)(_s?_s->chars:"<null>"));
	font_elements->yacHashGet(_s, &r);
	if(r.type==3)
	{
		yac_host->printf("    --> found.\n");
		//return dynamic_cast<FXFont*>(r.value.object_val);
		return *(FXFont**)&r.value.object_val;
	}
	return 0;
}

FXComposite *_TKS_FXPage::findCompositeElement(YAC_String*_s) {
	YAC_Value r;
	yac_host->printf("findCompositeElement(%s)\n", (char*)(_s?_s->chars:"<null>"));
	composite_elements->yacHashGet(_s, &r);
	if(r.type==3)
	{
		yac_host->printf("---> found\n");
		YAC_Object *o=r.value.object_val;
		FXComposite *com=(FXComposite*)(o);
		return com;
	}
	return 0;
}

void _TKS_FXPage::parseFXFont(YAC_Object *_h) {
	YAC_String *s;
	//sSI i;

	sSI i_sz=11;
	sSI i_weight=FONTWEIGHT_NORMAL;
	sSI i_slant=FONTSLANT_REGULAR;
	sSI i_enc=FONTENCODING_DEFAULT;
	sSI i_setwidth=FONTSETWIDTH_DONTCARE;
	sSI i_h=0;
	YAC_String s_defface; s_defface.visit("courier");
	YAC_String *s_face=&s_defface;

	_TKS_FXPage_GetHashIntVal(_h, &ats_size, &i_sz);
	_TKS_FXPage_GetHashIntVal(_h, &ats_h, &i_h);
	_TKS_FXPage_GetHashStringVal(_h, &ats_face, &s_face);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_weight, &s))
		i_weight=yac_host->yacScanFlags(s);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_slant, &s))
		i_slant=yac_host->yacScanFlags(s);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_encoding, &s))
		i_enc=yac_host->yacScanFlags(s);
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_setWidth, &s))
		i_enc=yac_host->yacScanFlags(s);

	yac_host->printf("[...] creating FXFont (face=%s sz=%i wt=%i sl=%i enc=%i sw=%i h=%i)\n",
		s_face->chars, i_sz, i_weight, i_slant, i_enc, i_setwidth, i_h);
	cobject.fxfont=new FXFont(application, s_face->chars, i_sz, i_weight, i_slant, i_enc, i_setwidth, i_h);

	YAC_Value r;
	_h->yacHashGet(&ats_id, &r);
	YAC_Value v;
	v.initObject(cobject.yacobject, 0);
	font_elements->yacHashSet(r.value.string_val, &v);
}

/*void _TKS_FXPage::parseID(YAC_Object *_h) {
	YAC_String *s;
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_id, &s))
	{
		YAC_Value v;
		v.initObject((YAC_Object*)cobject.fxobject, 0);
		elements->yacHashSet(s, &v);
	}
}*/

void _TKS_FXPage::parseCompositeID(YAC_Object *_h) {
	//fxprev.fxobject=cobject.fxobject;
	YAC_String *s;
	if(_TKS_FXPage_GetHashStringVal(_h, &ats_id, &s))
	{
		YAC_Value v;
		v.initObject((YAC_Object*)cobject.fxobject/*cobject_yac*/, 0);
		composite_elements->yacHashSet(s, &v);
		s_globals.append(cb_odecl);
		YAC_String ts; ts.visit(" fx_");
		s_globals.append(&cb_odecl_fxprefix);
		s_globals.append(s);
		s_globals.append(&cb_odecl_fxsuffix);
		//yac_host->printf("add id s_globals=%s fx=0x%08x\n", s_globals.chars, v.value.int_val);
	}
}

void _TKS_FXPage::parseNode(YAC_TreeNode*_t) {
	//cobject_yac=0;
	YAC_Object *h=_t->value.object_val;
	YAC_String *s;

	c_opts=0;
	b_defopt=1;

	YAC_Value r;
	h->yacHashGet(&ats_id, &r);
	c_sid=r.value.string_val;

	if(_TKS_FXPage_GetHashStringVal(h, &ats_flags, &s))
	{
		c_opts=yac_host->yacScanFlags(s);
		b_defopt=0;
	}
	
	c_forcedparent=0;
	if(_TKS_FXPage_GetHashStringVal(h, &ats_parent, &s))
		c_forcedparent=findCompositeElement(s);

	if(kw_fxmainwindow.compare(&_t->name))
	{
		cb_odecl=&cb_odecl_fxmainwindow;
		cobject.tksfxmainwindow=new TKS_FXMainWindow(b_defopt?DECOR_ALL:c_opts);
		cobject.tksfxmainwindow->class_ID=clid_FXMainWindow;

		//cobject_yac=(YAC_Object*)cobject.tksfxmainwindow;
		c_fxcomposite=cobject.tksfxmainwindow;
		parseFXWindowArgs(h);
		parseCompositeID(h);
		if(_TKS_FXPage_GetHashStringVal(h, &ats_title, &s))
			cobject.tksfxmainwindow->setTitle(s->chars);

		if(_t->right)
		{
			//yac_host->printf("parseFXMainWindow\n");
			pushFX();
			parseNode(_t->right);
			popFX();
			//yac_host->printf("/parseFXMainWindow\n");
		}
	}
	else if(kw_script.compare(&_t->name))
		parseScript(h);
	else if(kw_fxlabel.compare(&_t->name))
		parseFXLabel(h);
	else if(kw_fxbutton.compare(&_t->name))
		parseFXButton(h);
	else if(kw_fxfont.compare(&_t->name))
		parseFXFont(h);
	else if(kw_fxtextfield.compare(&_t->name))
		parseFXTextField(h);
	else if(kw_fxslider.compare(&_t->name))
		parseFXSlider(h);
	else if(kw_fxmenubar.compare(&_t->name))
		parseFXMenuBar(h,_t);
	else if(kw_fxmenupane.compare(&_t->name))
		parseFXMenuPane(h,_t);
	else if(kw_fxmenutitle.compare(&_t->name))
		parseFXMenuTitle(h);
	else if(kw_fxmenucommand.compare(&_t->name))
		parseFXMenuCommand(h);
	else if(kw_fxhorizontalframe.compare(&_t->name))
		parseFXHorizontalFrame(h,_t);
	else if(kw_fxverticalframe.compare(&_t->name))
		parseFXVerticalFrame(h,_t);
	else if(kw_fxdial.compare(&_t->name))
		parseFXDial(h);
	else if(kw_fxscrollbar.compare(&_t->name))
		parseFXScrollBar(h);
	else if(kw_fxgroupbox.compare(&_t->name))
		parseFXGroupBox(h,_t);
	else if(kw_fxhorizontalseparator.compare(&_t->name))
		parseFXHorizontalSeparator(h);
	else if(kw_fxverticalseparator.compare(&_t->name))
		parseFXVerticalSeparator(h);
	else if(kw_fxglcanvas.compare(&_t->name))
		parseFXGLCanvas(h);
	else if(kw_fxmessagebox.compare(&_t->name))
		parseFXMessageBox(h);
	else if(kw_fxfiledialog.compare(&_t->name))
		parseFXFileDialog(h);
	else if(kw_fx4splitter.compare(&_t->name))
		parseFX4Splitter(h,_t);
	else if(kw_fxcolordialog.compare(&_t->name))
		parseFXColorDialog(h);
	else if(kw_fxcheckbutton.compare(&_t->name))
		parseFXCheckButton(h);
	else if(kw_fxcombobox.compare(&_t->name))
		parseFXComboBox(h);
	else
	{
		yac_host->printf("[---] element \"%s\" is unknown.\n", _t->name.chars);
	}


	if(_t->left)
		parseNode(_t->left);
}

void _TKS_FXPage::tksinter_createFromTree(YAC_Object *_t, YAC_Value *_r) {
	if(YAC_CHK(_t, YAC_CLID_TREENODE))
	{
		YAC_TreeNode *t=(YAC_TreeNode*)_t;

		YAC_TreeNode *c=t;

		t_script.alloc(64*1024);
		t_script.length=1;

		s_globals.alloc(8*1024);
		s_globals.length=1;

		if(elements)
			yac_host->yacDelete(elements);
		elements=yac_host->yacNewByID(YAC_CLID_HASHTABLE);
		elements->yacArrayAlloc(4096,1,1,4);

		if(font_elements)
			yac_host->yacDelete(font_elements);
		font_elements=yac_host->yacNewByID(YAC_CLID_HASHTABLE);
		font_elements->yacArrayAlloc(256,1,1,4);

		if(composite_elements)
			yac_host->yacDelete(composite_elements);
		composite_elements=yac_host->yacNewByID(YAC_CLID_HASHTABLE);
		composite_elements->yacArrayAlloc(4096,1,1,4);

		kw_unset.visit("<<unset>>");
		sfx_unset="<<unset>>";

		kw_script.visit("script");

		kw_fxmainwindow.visit("fxmainwindow");
		kw_fxlabel.visit("fxlabel");
		kw_fxbutton.visit("fxbutton");
		kw_fxfont.visit("fxfont");
		kw_fxtextfield.visit("fxtextfield");
		kw_fxslider.visit("fxslider");
		kw_fxmenubar.visit("fxmenubar");
		kw_fxmenupane.visit("fxmenupane");
		kw_fxmenutitle.visit("fxmenutitle");
		kw_fxmenucommand.visit("fxmenucommand");
		kw_fxhorizontalframe.visit("fxhorizontalframe");
		kw_fxverticalframe.visit("fxverticalframe");
		kw_fxdial.visit("fxdial");
		kw_fxscrollbar.visit("fxscrollbar");
		kw_fxgroupbox.visit("fxgroupbox");
		kw_fxhorizontalseparator.visit("fxhorizontalseparator");
		kw_fxverticalseparator.visit("fxverticalseparator");
		kw_fxglcanvas.visit("fxglcanvas");
		kw_fxmessagebox.visit("fxmessagebox");
		kw_fxfiledialog.visit("fxfiledialog");
		kw_fx4splitter.visit("fx4splitter");
		kw_fxcolordialog.visit("fxcolordialog");
		kw_fxcheckbutton.visit("fxcheckbutton");
		kw_fxcombobox.visit("fxcombobox");

		cbs_onUpdate.visit("onUpdate");
		cbs_onEnter.visit("onEnter");
		cbs_onLeave.visit("onLeave");
		cbs_onFocusIn.visit("onFocusIn");
		cbs_onFocusOut.visit("onFocusOut");
		cbs_onUngrabbed.visit("onUngrabbed");
		cbs_onLeftBtnPress.visit("onLeftBtnPress");
		cbs_onLeftBtnRelease.visit("onLeftBtnRelease");
		cbs_onKeyPress.visit("onKeyPress");
		cbs_onKeyRelease.visit("onKeyRelease");
		cbs_onHotKeyPress.visit("onHotKeyPress");
		cbs_onHotKeyRelease.visit("onHotKeyRelease");
		cbs_onCheck.visit("onCheck");
		cbs_onUncheck.visit("onUncheck");
		cbs_onVerify.visit("onVerify");
		cbs_onMotion.visit("onMotion");
		cbs_onTimeout.visit("onTimeout");
		cbs_onPaint.visit("onPaint");
		cbs_onRightBtnRelease.visit("onRightBtnRelease");
		cbs_onListClicked.visit("onListClicked");

		ats_title.visit("title");
		ats_x.visit("x");
		ats_y.visit("y");
		ats_w.visit("w");
		ats_h.visit("h");
		ats_hSpacing.visit("hSpacing");
		ats_vSpacing.visit("vSpacing");
		ats_padLeft.visit("padLeft");
		ats_padRight.visit("padRight");
		ats_padTop.visit("padTop");
		ats_padBottom.visit("padBottom");
		ats_icon.visit("icon");
		ats_packingHints.visit("packingHints");
		ats_text.visit("text");
		ats_helpText.visit("helpText");
		ats_tipText.visit("tipText");
		ats_justify.visit("justify");
		ats_iconPosition.visit("iconPosition");
		ats_frameStyle.visit("frameStyle");
		ats_layoutHints.visit("layoutHints");
		ats_flags.visit("flags");
		ats_id.visit("id");
		ats_size.visit("size");
		ats_face.visit("face");
		ats_weight.visit("weight");
		ats_slant.visit("slant");
		ats_encoding.visit("encoding");
		ats_setWidth.visit("setWidth");
		ats_font.visit("font");
		ats_cols.visit("cols");
		ats_orientation.visit("orientation");
		ats_headSize.visit("headSize");
		ats_slotSize.visit("slotSize");
		ats_lo.visit("lo");
		ats_hi.visit("hi");
		ats_value.visit("value");
		ats_inc.visit("inc");
		ats_tickDelta.visit("tickDelta");
		ats_label.visit("label");
		ats_parent.visit("parent");
		ats_revolutionIncrement.visit("revolutionIncrement");
		ats_notchSpacing.visit("notchSpacing");
		ats_notchOffset.visit("notchOffset");
		ats_dialStyle.visit("dialStyle");
		ats_page.visit("page");
		ats_groupBoxStyle.visit("groupBoxStyle");
		ats_timeout.visit("timeout");
		ats_body.visit("body");
		ats_caption.visit("caption");
		ats_message.visit("message");
		ats_filename.visit("filename");
		ats_pattern.visit("pattern");
		ats_patternList.visit("patternList");
		ats_currentPattern.visit("currentPattern");
		ats_directory.visit("directory");
		ats_itemSpace.visit("itemSpace");
		ats_fileBoxStyle.visit("fileBoxStyle");
		ats_selectMode.visit("selectMode");
		ats_showReadOnly.visit("showReadOnly");
		ats_hSplit.visit("hSplit");
		ats_vSplit.visit("vSplit");
		ats_splitterStyle.visit("splitterStyle");
		ats_barSize.visit("barSize");
		ats_expanded.visit("expanded");
		ats_rgba.visit("rgba");
		ats_opaqueOnly.visit("opaqueOnly");
		ats_check.visit("check");
		ats_checkButtonStyle.visit("checkButtonStyle");
		ats_comboStyle.visit("comboStyle");
		ats_numVisible.visit("numVisible");
		ats_currentItem.visit("currentItem");
		ats_editable.visit("editable");
		ats_numColumns.visit("numColumns");
		ats_items.visit("items");

		os_function.visit("function ");
		os_voidargs.visit("() {\n");
		//os_stringargs.visit("(String _s) {\n");
		//os_intargs.visit("(int _i) {\n");
		//os_intintargs.visit("(int _i, int _j) {\n");
		//os_floatargs.visit("(float _f) {\n");
		os_endfunction.visit("\n}\n");

		cb_args_fxtextfield_onVerify.visit("(String _text) {");
		cb_args_fxtextfield_onKey.visit("(int _code, int _state) {");
		cb_args_fxslider_onChange.visit("(int _value) {");
		cb_args_fxglcanvas_onMotion.visit("(int _x, int _y) {");
		cb_args_fxcheckbutton_onLeftBtnRelease.visit("(int _check) {");
		cb_args_fxcombobox_onListClicked.visit("(int _item) {");

		cobject.fxobject=0;
		c_forcedparent=0;
		c_sid=0;

		pushFX();
		parseNode(t);
		popFX();

		
		YAC_String ts;
		ts.alloc(s_globals.length+t_script.length);
		ts.length=1;
		ts.append(&s_globals);
		sUI i;
		for(i=0; i<script_tag_index; i++)
			ts.append(script_tags[i]);
		ts.append(&t_script);
		t_script.free();
		s_globals.free();
		if(yac_host->yacGetDebugLevel())
		{
			yac_host->printf("\n--------------------------------------------\nscript=\"");
			yac_host->printf((char*)ts.chars);
			yac_host->printf("\"\n--------------------------------------------\n");
		}

		script=yac_host->yacCompileModule(ts.chars);
		if(script)
		{
			// ---- relocate ----
			FXPageRelocEntry*r=reloc;
			for(i=0; i<reloci; i++)
			{
				//yac_host->printf("relocfun(%s) object=%08x clid=%i\n",r->fn_name.chars, (sUI)r->fx_object, r->fx_object->class_ID);
				void *fn=yac_host->yacFindFunctionInModule(script, (char*)r->fn_name.chars);
				if(!fn)
				{
					yac_host->printf("[---] reloc[%i::%s] failed.\n",i,(char*)r->fn_name.chars);
					YAC_RETI(0);
					return;
				}
				*r->sc_function=fn;
				void *var=yac_host->yacFindVariableInFunction(fn, "fx");
				if(var)
				{
					YAC_Value vfx;
					vfx.initObject(r->fx_object, 0);
					//yac_host->printf("[...] fx=0x%08x\n", (sUI)r->fx_object);
					yac_host->yacSetVariable(var, &vfx);
				}
				else
				{
					yac_host->printf("[---] internal error: fx object not found.\n");
				}
				r++;
			}
			//yac_host->printf("[...] relocated %i functions.\n",i);
			YAC_Iterator *it=composite_elements->yacGetIterator();
			it->begin();
			YAC_Value itv;
			YAC_String vname; vname.alloc(1024); 
			do 
			{
				itv=&it->getNext();
				if(itv.type==4)
					if(itv.value.string_val)
					{
						vname.length=1;
						vname.append(&cb_odecl_fxprefix2);
						vname.append(itv.value.string_val);
						void *var=yac_host->yacFindVariableInModule(script, vname.chars);
						if(var)
						{
							YAC_Value vobj;
							composite_elements->yacHashGet(itv.value.string_val, &vobj);
							yac_host->yacSetVariable(var, &vobj);
							//yac_host->printf("[...] set fxobject id \"%s\" adr=0x%08x\n", vname.chars, (sUI)vobj.value.object_val);
						}
						else
						{
							yac_host->printf("[---] internal error: fx_ object not found s=\"%s\".\n", vname.chars);
						}
					}
			} while(itv.type==4);
			it->end();
	
			delete it;
		}

		ts.free();

		YAC_RETI(1);
		return;
	}
	YAC_RETI(0);
}



// ---------------------------------------------------------------------------------------------
YAC_STemplate <TKS_FXApp>        *t_FXApp;
YAC_Template  <_TKS_FXPage>      *t_FXPage;
YAC_RTemplate <TKS_FXMainWindow> *t_FXMainWindow;

YAC_RTemplate <TKS_FXButton>     *t_FXButton;
YAC_RTemplate <TKS_FXDial>       *t_FXDial;
YAC_RTemplate <TKS_FXSlider>     *t_FXSlider;
YAC_RTemplate <TKS_FXScrollBar>  *t_FXScrollBar;
YAC_RTemplate <TKS_FXTextField>  *t_FXTextField;
YAC_RTemplate <TKS_FXGLCanvas>   *t_FXGLCanvas;
YAC_RTemplate <TKS_FXLabel>      *t_FXLabel;
YAC_RTemplate <TKS_FXMessageBox> *t_FXMessageBox;
YAC_RTemplate <TKS_FXFileDialog> *t_FXFileDialog;
YAC_RTemplate <TKS_FX4Splitter>  *t_FX4Splitter;
YAC_RTemplate <TKS_FXColorDialog>*t_FXColorDialog;
YAC_RTemplate <TKS_FXCheckButton>*t_FXCheckButton;
YAC_RTemplate <TKS_FXComboBox>   *t_FXComboBox;

//sUI                                clid_FXMenuCommand;

static char *s_argv[]={"test"};

YAC_APIC void YAC_Init(YAC_Host *_host) {
	yac_host=_host;
	
	application=new FXApp();
	int argc=1;
	application->init(argc,s_argv);

	fxglvisual=0;

	t_FXApp        =new YAC_STemplate <TKS_FXApp>        (_host);
	t_FXPage       =new YAC_Template  <_TKS_FXPage>      (_host);
	t_FXMainWindow =new YAC_RTemplate <TKS_FXMainWindow> (_host);
	t_FXLabel      =new YAC_RTemplate <TKS_FXLabel>      (_host);
	t_FXButton     =new YAC_RTemplate <TKS_FXButton>     (_host);
	t_FXSlider     =new YAC_RTemplate <TKS_FXSlider>     (_host);
	t_FXDial       =new YAC_RTemplate <TKS_FXDial>       (_host);
	t_FXScrollBar  =new YAC_RTemplate <TKS_FXScrollBar>  (_host);
	t_FXTextField  =new YAC_RTemplate <TKS_FXTextField>  (_host);
	t_FXGLCanvas   =new YAC_RTemplate <TKS_FXGLCanvas>   (_host);
	t_FXMessageBox =new YAC_RTemplate <TKS_FXMessageBox> (_host);
	t_FXFileDialog =new YAC_RTemplate <TKS_FXFileDialog> (_host);
	t_FX4Splitter  =new YAC_RTemplate <TKS_FX4Splitter>  (_host);
	t_FXColorDialog=new YAC_RTemplate <TKS_FXColorDialog>(_host);
	t_FXCheckButton=new YAC_RTemplate <TKS_FXCheckButton>(_host);
	t_FXComboBox   =new YAC_RTemplate <TKS_FXComboBox>   (_host);

	clid_FXPage       =t_FXPage        ->ctemplate->class_ID;
	clid_FXMainWindow =t_FXMainWindow  ->ctemplate->class_ID;
	clid_FXLabel      =t_FXLabel       ->ctemplate->class_ID;
	clid_FXButton     =t_FXButton      ->ctemplate->class_ID;
	clid_FXDial       =t_FXDial        ->ctemplate->class_ID;
	clid_FXSlider     =t_FXSlider      ->ctemplate->class_ID;
	clid_FXScrollBar  =t_FXScrollBar   ->ctemplate->class_ID;
	clid_FXTextField  =t_FXTextField   ->ctemplate->class_ID;
	clid_FXMessageBox =t_FXMessageBox  ->ctemplate->class_ID;
	clid_FXFileDialog =t_FXFileDialog  ->ctemplate->class_ID;
	clid_FX4Splitter  =t_FX4Splitter   ->ctemplate->class_ID;
	clid_FXColorDialog=t_FXColorDialog ->ctemplate->class_ID;
	clid_FXCheckButton=t_FXCheckButton ->ctemplate->class_ID;
	clid_FXComboBox   =t_FXComboBox    ->ctemplate->class_ID;

	cb_odecl_fxmainwindow.visit("FXMainWindow");
	cb_odecl_fxbutton.visit("FXButton");
	cb_odecl_fxdial.visit("FXDial");
	cb_odecl_fxslider.visit("FXSlider");
	cb_odecl_fxscrollbar.visit("FXScrollBar");
	cb_odecl_fxtextfield.visit("FXTextField");
	cb_odecl_fxglcanvas.visit("FXGLCanvas");
	cb_odecl_fxlabel.visit("FXLabel");
	cb_odecl_fxmessagebox.visit("FXMessageBox");
	cb_odecl_fxfiledialog.visit("FXFileDialog");
	cb_odecl_fx4splitter.visit("FX4Splitter");
	cb_odecl_fxcolordialog.visit("FXColorDialog");
	cb_odecl_fxcheckbutton.visit("FXCheckButton");
	cb_odecl_fxcombobox.visit("FXComboBox");

	cb_odecl_fxname.visit(" fx;");
	cb_odecl_fxprefix.visit(" fx_");
	cb_odecl_fxsuffix.visit(";\n");
	cb_odecl_fxprefix2.visit("fx_");
}	

YAC_APIC void YAC_Exit(YAC_Host *_host) {
	//printf("fox::YAC_Exit1\n");
	delete t_FXPage;
	//printf("fox::~YAC_Exit2\n");
	delete t_FXComboBox;
	//printf("fox::~YAC_Exit3\n");
	delete t_FXCheckButton;
	//printf("fox::~YAC_Exit4\n");
	delete t_FXColorDialog;
	//printf("fox::~YAC_Exit5\n");
	delete t_FX4Splitter;
	//printf("fox::~YAC_Exit6\n");
	delete t_FXFileDialog;
	//printf("fox::~YAC_Exit7\n");
	delete t_FXMessageBox;
	//printf("fox::~YAC_Exit8\n");
	delete t_FXGLCanvas;
	//printf("fox::~YAC_Exit9\n");
	delete t_FXTextField;
	//printf("fox::~YAC_Exit10\n");
	delete t_FXScrollBar;
	//printf("fox::~YAC_Exit11\n");
	delete t_FXDial;
	//printf("fox::~YAC_Exit12\n");
	delete t_FXSlider;
	//printf("fox::~YAC_Exit13\n");
	delete t_FXButton;
	//printf("fox::~YAC_Exit14\n");
	delete t_FXLabel;
	//printf("fox::~YAC_Exit15\n");
	delete t_FXMainWindow;
	//printf("fox::~YAC_Exit16\n");
	if(fxglvisual)
	{
		delete fxglvisual;
		fxglvisual=0;
	}
	delete t_FXApp;
	if(application)
	{
		delete application;
		application=0;
	}
}

sUI YAC_Version(void) {
	return 0x00070001;
}


YAC_APIC const sChar *YAC_GetFunctionStringList(void) { 
    return 
	     "FXMessageBox_error(oioo):i "
	     "FXMessageBox_warning(oioo):i "
	     "FXMessageBox_question(oioo):i "
	     "FXMessageBox_information(oioo):i "
		 ;
}


